// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using Microsoft.Azure;
using Microsoft.WindowsAzure.Management.Compute;
using Microsoft.WindowsAzure.Management.Compute.Models;

namespace Microsoft.WindowsAzure.Management.Compute
{
    /// <summary>
    /// The Service Management API provides programmatic access to much of the
    /// functionality available through the Management Portal. The Service
    /// Management API is a REST API. All API operations are performed over
    /// SSL, and are mutually authenticated using X.509 v3 certificates.  (see
    /// http://msdn.microsoft.com/en-us/library/windowsazure/ee460799.aspx for
    /// more information)
    /// </summary>
    public static partial class VirtualMachineVMImageOperationsExtensions
    {
        /// <summary>
        /// The Create VM Image operation creates a VM image that in your image
        /// repository.  (see
        /// http://msdn.microsoft.com/en-us/library/azure/dn775054.aspx for
        /// more information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Compute.IVirtualMachineVMImageOperations.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the virtual machine VM image
        /// create operation.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static AzureOperationResponse BeginCreating(this IVirtualMachineVMImageOperations operations, VirtualMachineVMImageCreateParameters parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IVirtualMachineVMImageOperations)s).BeginCreatingAsync(parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The Create VM Image operation creates a VM image that in your image
        /// repository.  (see
        /// http://msdn.microsoft.com/en-us/library/azure/dn775054.aspx for
        /// more information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Compute.IVirtualMachineVMImageOperations.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the virtual machine VM image
        /// create operation.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<AzureOperationResponse> BeginCreatingAsync(this IVirtualMachineVMImageOperations operations, VirtualMachineVMImageCreateParameters parameters)
        {
            return operations.BeginCreatingAsync(parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// The Begin Deleting Virtual Machine Image operation deletes the
        /// specified virtual machine image.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Compute.IVirtualMachineVMImageOperations.
        /// </param>
        /// <param name='vmImageName'>
        /// Required. The name of the virtual machine image to delete.
        /// </param>
        /// <param name='deleteFromStorage'>
        /// Required. Specifies that the source blob for the image should also
        /// be deleted from storage.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static AzureOperationResponse BeginDeleting(this IVirtualMachineVMImageOperations operations, string vmImageName, bool deleteFromStorage)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IVirtualMachineVMImageOperations)s).BeginDeletingAsync(vmImageName, deleteFromStorage);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The Begin Deleting Virtual Machine Image operation deletes the
        /// specified virtual machine image.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Compute.IVirtualMachineVMImageOperations.
        /// </param>
        /// <param name='vmImageName'>
        /// Required. The name of the virtual machine image to delete.
        /// </param>
        /// <param name='deleteFromStorage'>
        /// Required. Specifies that the source blob for the image should also
        /// be deleted from storage.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<AzureOperationResponse> BeginDeletingAsync(this IVirtualMachineVMImageOperations operations, string vmImageName, bool deleteFromStorage)
        {
            return operations.BeginDeletingAsync(vmImageName, deleteFromStorage, CancellationToken.None);
        }
        
        /// <summary>
        /// Share an already replicated VM image. This operation is only for
        /// publishers. You have to be registered as image publisher with
        /// Windows Azure to be able to call this.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Compute.IVirtualMachineVMImageOperations.
        /// </param>
        /// <param name='vmImageName'>
        /// Required. The name of the virtual machine image to share.
        /// </param>
        /// <param name='permission'>
        /// Required. The sharing permission: public, msdn, or private.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static AzureOperationResponse BeginSharing(this IVirtualMachineVMImageOperations operations, string vmImageName, string permission)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IVirtualMachineVMImageOperations)s).BeginSharingAsync(vmImageName, permission);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Share an already replicated VM image. This operation is only for
        /// publishers. You have to be registered as image publisher with
        /// Windows Azure to be able to call this.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Compute.IVirtualMachineVMImageOperations.
        /// </param>
        /// <param name='vmImageName'>
        /// Required. The name of the virtual machine image to share.
        /// </param>
        /// <param name='permission'>
        /// Required. The sharing permission: public, msdn, or private.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<AzureOperationResponse> BeginSharingAsync(this IVirtualMachineVMImageOperations operations, string vmImageName, string permission)
        {
            return operations.BeginSharingAsync(vmImageName, permission, CancellationToken.None);
        }
        
        /// <summary>
        /// Unreplicate an VM image to multiple target locations. This
        /// operation is only for publishers. You have to be registered as
        /// image publisher with Windows Azure to be able to call this. Note:
        /// The operation removes the published copies of the user VM Image.
        /// It does not remove the actual user VM Image. To remove the actual
        /// user VM Image, the publisher will have to call Delete VM Image.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Compute.IVirtualMachineVMImageOperations.
        /// </param>
        /// <param name='vmImageName'>
        /// Required. The name of the virtual machine image to replicate. Note:
        /// The VM Image Name should be the user VM Image, not the published
        /// name of the VM Image.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static AzureOperationResponse BeginUnreplicating(this IVirtualMachineVMImageOperations operations, string vmImageName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IVirtualMachineVMImageOperations)s).BeginUnreplicatingAsync(vmImageName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Unreplicate an VM image to multiple target locations. This
        /// operation is only for publishers. You have to be registered as
        /// image publisher with Windows Azure to be able to call this. Note:
        /// The operation removes the published copies of the user VM Image.
        /// It does not remove the actual user VM Image. To remove the actual
        /// user VM Image, the publisher will have to call Delete VM Image.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Compute.IVirtualMachineVMImageOperations.
        /// </param>
        /// <param name='vmImageName'>
        /// Required. The name of the virtual machine image to replicate. Note:
        /// The VM Image Name should be the user VM Image, not the published
        /// name of the VM Image.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<AzureOperationResponse> BeginUnreplicatingAsync(this IVirtualMachineVMImageOperations operations, string vmImageName)
        {
            return operations.BeginUnreplicatingAsync(vmImageName, CancellationToken.None);
        }
        
        /// <summary>
        /// The Create VM Image operation creates a VM image that in your image
        /// repository.  (see
        /// http://msdn.microsoft.com/en-us/library/azure/dn775054.aspx for
        /// more information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Compute.IVirtualMachineVMImageOperations.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the Create Virtual Machine Image
        /// operation.
        /// </param>
        /// <returns>
        /// The response body contains the status of the specified asynchronous
        /// operation, indicating whether it has succeeded, is inprogress, or
        /// has failed. Note that this status is distinct from the HTTP status
        /// code returned for the Get Operation Status operation itself. If
        /// the asynchronous operation succeeded, the response body includes
        /// the HTTP status code for the successful request. If the
        /// asynchronous operation failed, the response body includes the HTTP
        /// status code for the failed request and error information regarding
        /// the failure.
        /// </returns>
        public static OperationStatusResponse Create(this IVirtualMachineVMImageOperations operations, VirtualMachineVMImageCreateParameters parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IVirtualMachineVMImageOperations)s).CreateAsync(parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The Create VM Image operation creates a VM image that in your image
        /// repository.  (see
        /// http://msdn.microsoft.com/en-us/library/azure/dn775054.aspx for
        /// more information)
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Compute.IVirtualMachineVMImageOperations.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the Create Virtual Machine Image
        /// operation.
        /// </param>
        /// <returns>
        /// The response body contains the status of the specified asynchronous
        /// operation, indicating whether it has succeeded, is inprogress, or
        /// has failed. Note that this status is distinct from the HTTP status
        /// code returned for the Get Operation Status operation itself. If
        /// the asynchronous operation succeeded, the response body includes
        /// the HTTP status code for the successful request. If the
        /// asynchronous operation failed, the response body includes the HTTP
        /// status code for the failed request and error information regarding
        /// the failure.
        /// </returns>
        public static Task<OperationStatusResponse> CreateAsync(this IVirtualMachineVMImageOperations operations, VirtualMachineVMImageCreateParameters parameters)
        {
            return operations.CreateAsync(parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// The Delete Virtual Machine Image operation deletes the specified
        /// virtual machine image.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Compute.IVirtualMachineVMImageOperations.
        /// </param>
        /// <param name='vmImageName'>
        /// Required. The name of the virtual machine image to delete.
        /// </param>
        /// <param name='deleteFromStorage'>
        /// Required. Specifies that the source blob for the image should also
        /// be deleted from storage.
        /// </param>
        /// <returns>
        /// The response body contains the status of the specified asynchronous
        /// operation, indicating whether it has succeeded, is inprogress, or
        /// has failed. Note that this status is distinct from the HTTP status
        /// code returned for the Get Operation Status operation itself. If
        /// the asynchronous operation succeeded, the response body includes
        /// the HTTP status code for the successful request. If the
        /// asynchronous operation failed, the response body includes the HTTP
        /// status code for the failed request and error information regarding
        /// the failure.
        /// </returns>
        public static OperationStatusResponse Delete(this IVirtualMachineVMImageOperations operations, string vmImageName, bool deleteFromStorage)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IVirtualMachineVMImageOperations)s).DeleteAsync(vmImageName, deleteFromStorage);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The Delete Virtual Machine Image operation deletes the specified
        /// virtual machine image.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Compute.IVirtualMachineVMImageOperations.
        /// </param>
        /// <param name='vmImageName'>
        /// Required. The name of the virtual machine image to delete.
        /// </param>
        /// <param name='deleteFromStorage'>
        /// Required. Specifies that the source blob for the image should also
        /// be deleted from storage.
        /// </param>
        /// <returns>
        /// The response body contains the status of the specified asynchronous
        /// operation, indicating whether it has succeeded, is inprogress, or
        /// has failed. Note that this status is distinct from the HTTP status
        /// code returned for the Get Operation Status operation itself. If
        /// the asynchronous operation succeeded, the response body includes
        /// the HTTP status code for the successful request. If the
        /// asynchronous operation failed, the response body includes the HTTP
        /// status code for the failed request and error information regarding
        /// the failure.
        /// </returns>
        public static Task<OperationStatusResponse> DeleteAsync(this IVirtualMachineVMImageOperations operations, string vmImageName, bool deleteFromStorage)
        {
            return operations.DeleteAsync(vmImageName, deleteFromStorage, CancellationToken.None);
        }
        
        /// <summary>
        /// Gets VMImage's properties and its replication details. This
        /// operation is only for publishers. You have to be registered as
        /// image publisher with Windows Azure to be able to call this.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Compute.IVirtualMachineVMImageOperations.
        /// </param>
        /// <param name='vmImageName'>
        /// Required. The name of the virtual machine image to replicate.
        /// </param>
        /// <returns>
        /// The Get Details VM Images operation response.
        /// </returns>
        public static VirtualMachineVMImageGetDetailsResponse GetDetails(this IVirtualMachineVMImageOperations operations, string vmImageName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IVirtualMachineVMImageOperations)s).GetDetailsAsync(vmImageName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Gets VMImage's properties and its replication details. This
        /// operation is only for publishers. You have to be registered as
        /// image publisher with Windows Azure to be able to call this.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Compute.IVirtualMachineVMImageOperations.
        /// </param>
        /// <param name='vmImageName'>
        /// Required. The name of the virtual machine image to replicate.
        /// </param>
        /// <returns>
        /// The Get Details VM Images operation response.
        /// </returns>
        public static Task<VirtualMachineVMImageGetDetailsResponse> GetDetailsAsync(this IVirtualMachineVMImageOperations operations, string vmImageName)
        {
            return operations.GetDetailsAsync(vmImageName, CancellationToken.None);
        }
        
        /// <summary>
        /// The List Virtual Machine Images operation retrieves a list of the
        /// virtual machine images.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Compute.IVirtualMachineVMImageOperations.
        /// </param>
        /// <returns>
        /// The List VM Images operation response.
        /// </returns>
        public static VirtualMachineVMImageListResponse List(this IVirtualMachineVMImageOperations operations)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IVirtualMachineVMImageOperations)s).ListAsync();
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The List Virtual Machine Images operation retrieves a list of the
        /// virtual machine images.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Compute.IVirtualMachineVMImageOperations.
        /// </param>
        /// <returns>
        /// The List VM Images operation response.
        /// </returns>
        public static Task<VirtualMachineVMImageListResponse> ListAsync(this IVirtualMachineVMImageOperations operations)
        {
            return operations.ListAsync(CancellationToken.None);
        }
        
        /// <summary>
        /// Replicate an VM image to multiple target locations. This operation
        /// is only for publishers. You have to be registered as image
        /// publisher with Windows Azure to be able to call this.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Compute.IVirtualMachineVMImageOperations.
        /// </param>
        /// <param name='vmImageName'>
        /// Required. The name of the virtual machine image to replicate.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the Replicate Virtual Machine
        /// Image operation.
        /// </param>
        /// <returns>
        /// The response body contains the published name of the image.
        /// </returns>
        public static VirtualMachineVMImageReplicateResponse Replicate(this IVirtualMachineVMImageOperations operations, string vmImageName, VirtualMachineVMImageReplicateParameters parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IVirtualMachineVMImageOperations)s).ReplicateAsync(vmImageName, parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Replicate an VM image to multiple target locations. This operation
        /// is only for publishers. You have to be registered as image
        /// publisher with Windows Azure to be able to call this.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Compute.IVirtualMachineVMImageOperations.
        /// </param>
        /// <param name='vmImageName'>
        /// Required. The name of the virtual machine image to replicate.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the Replicate Virtual Machine
        /// Image operation.
        /// </param>
        /// <returns>
        /// The response body contains the published name of the image.
        /// </returns>
        public static Task<VirtualMachineVMImageReplicateResponse> ReplicateAsync(this IVirtualMachineVMImageOperations operations, string vmImageName, VirtualMachineVMImageReplicateParameters parameters)
        {
            return operations.ReplicateAsync(vmImageName, parameters, CancellationToken.None);
        }
        
        /// <summary>
        /// Share an already replicated VM image. This operation is only for
        /// publishers. You have to be registered as image publisher with
        /// Windows Azure to be able to call this.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Compute.IVirtualMachineVMImageOperations.
        /// </param>
        /// <param name='vmImageName'>
        /// Required. The name of the virtual machine image to share.
        /// </param>
        /// <param name='permission'>
        /// Required. The sharing permission: public, msdn, or private.
        /// </param>
        /// <returns>
        /// The response body contains the status of the specified asynchronous
        /// operation, indicating whether it has succeeded, is inprogress, or
        /// has failed. Note that this status is distinct from the HTTP status
        /// code returned for the Get Operation Status operation itself. If
        /// the asynchronous operation succeeded, the response body includes
        /// the HTTP status code for the successful request. If the
        /// asynchronous operation failed, the response body includes the HTTP
        /// status code for the failed request and error information regarding
        /// the failure.
        /// </returns>
        public static OperationStatusResponse Share(this IVirtualMachineVMImageOperations operations, string vmImageName, string permission)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IVirtualMachineVMImageOperations)s).ShareAsync(vmImageName, permission);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Share an already replicated VM image. This operation is only for
        /// publishers. You have to be registered as image publisher with
        /// Windows Azure to be able to call this.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Compute.IVirtualMachineVMImageOperations.
        /// </param>
        /// <param name='vmImageName'>
        /// Required. The name of the virtual machine image to share.
        /// </param>
        /// <param name='permission'>
        /// Required. The sharing permission: public, msdn, or private.
        /// </param>
        /// <returns>
        /// The response body contains the status of the specified asynchronous
        /// operation, indicating whether it has succeeded, is inprogress, or
        /// has failed. Note that this status is distinct from the HTTP status
        /// code returned for the Get Operation Status operation itself. If
        /// the asynchronous operation succeeded, the response body includes
        /// the HTTP status code for the successful request. If the
        /// asynchronous operation failed, the response body includes the HTTP
        /// status code for the failed request and error information regarding
        /// the failure.
        /// </returns>
        public static Task<OperationStatusResponse> ShareAsync(this IVirtualMachineVMImageOperations operations, string vmImageName, string permission)
        {
            return operations.ShareAsync(vmImageName, permission, CancellationToken.None);
        }
        
        /// <summary>
        /// Unreplicate an VM image to multiple target locations. This
        /// operation is only for publishers. You have to be registered as
        /// image publisher with Windows Azure to be able to call this. Note:
        /// The operation removes the published copies of the user VM Image.
        /// It does not remove the actual user VM Image. To remove the actual
        /// user VM Image, the publisher will have to call Delete VM Image.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Compute.IVirtualMachineVMImageOperations.
        /// </param>
        /// <param name='vmImageName'>
        /// Required. The name of the virtual machine image to replicate. Note:
        /// The VM Image Name should be the user VM Image, not the published
        /// name of the VM Image.
        /// </param>
        /// <returns>
        /// The response body contains the status of the specified asynchronous
        /// operation, indicating whether it has succeeded, is inprogress, or
        /// has failed. Note that this status is distinct from the HTTP status
        /// code returned for the Get Operation Status operation itself. If
        /// the asynchronous operation succeeded, the response body includes
        /// the HTTP status code for the successful request. If the
        /// asynchronous operation failed, the response body includes the HTTP
        /// status code for the failed request and error information regarding
        /// the failure.
        /// </returns>
        public static OperationStatusResponse Unreplicate(this IVirtualMachineVMImageOperations operations, string vmImageName)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IVirtualMachineVMImageOperations)s).UnreplicateAsync(vmImageName);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// Unreplicate an VM image to multiple target locations. This
        /// operation is only for publishers. You have to be registered as
        /// image publisher with Windows Azure to be able to call this. Note:
        /// The operation removes the published copies of the user VM Image.
        /// It does not remove the actual user VM Image. To remove the actual
        /// user VM Image, the publisher will have to call Delete VM Image.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Compute.IVirtualMachineVMImageOperations.
        /// </param>
        /// <param name='vmImageName'>
        /// Required. The name of the virtual machine image to replicate. Note:
        /// The VM Image Name should be the user VM Image, not the published
        /// name of the VM Image.
        /// </param>
        /// <returns>
        /// The response body contains the status of the specified asynchronous
        /// operation, indicating whether it has succeeded, is inprogress, or
        /// has failed. Note that this status is distinct from the HTTP status
        /// code returned for the Get Operation Status operation itself. If
        /// the asynchronous operation succeeded, the response body includes
        /// the HTTP status code for the successful request. If the
        /// asynchronous operation failed, the response body includes the HTTP
        /// status code for the failed request and error information regarding
        /// the failure.
        /// </returns>
        public static Task<OperationStatusResponse> UnreplicateAsync(this IVirtualMachineVMImageOperations operations, string vmImageName)
        {
            return operations.UnreplicateAsync(vmImageName, CancellationToken.None);
        }
        
        /// <summary>
        /// The Update VM Image operation updates a VM image that in your image
        /// repository.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Compute.IVirtualMachineVMImageOperations.
        /// </param>
        /// <param name='imageName'>
        /// Required. The name of the virtual machine image to be updated.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the Update Virtual Machine Image
        /// operation.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static AzureOperationResponse Update(this IVirtualMachineVMImageOperations operations, string imageName, VirtualMachineVMImageUpdateParameters parameters)
        {
            return Task.Factory.StartNew((object s) => 
            {
                return ((IVirtualMachineVMImageOperations)s).UpdateAsync(imageName, parameters);
            }
            , operations, CancellationToken.None, TaskCreationOptions.None, TaskScheduler.Default).Unwrap().GetAwaiter().GetResult();
        }
        
        /// <summary>
        /// The Update VM Image operation updates a VM image that in your image
        /// repository.
        /// </summary>
        /// <param name='operations'>
        /// Reference to the
        /// Microsoft.WindowsAzure.Management.Compute.IVirtualMachineVMImageOperations.
        /// </param>
        /// <param name='imageName'>
        /// Required. The name of the virtual machine image to be updated.
        /// </param>
        /// <param name='parameters'>
        /// Required. Parameters supplied to the Update Virtual Machine Image
        /// operation.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public static Task<AzureOperationResponse> UpdateAsync(this IVirtualMachineVMImageOperations operations, string imageName, VirtualMachineVMImageUpdateParameters parameters)
        {
            return operations.UpdateAsync(imageName, parameters, CancellationToken.None);
        }
    }
}
