// 
// Copyright (c) Microsoft and contributors.  All rights reserved.
// 
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//   http://www.apache.org/licenses/LICENSE-2.0
// 
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// 
// See the License for the specific language governing permissions and
// limitations under the License.
// 

// Warning: This code was generated by a tool.
// 
// Changes to this file may cause incorrect behavior and will be lost if the
// code is regenerated.

using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Net;
using System.Net.Http;
using System.Net.Http.Headers;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Xml.Linq;
using Hyak.Common;
using Hyak.Common.Internals;
using Microsoft.Azure;
using Microsoft.WindowsAzure.Management.ServiceBus;
using Microsoft.WindowsAzure.Management.ServiceBus.Models;

namespace Microsoft.WindowsAzure.Management.ServiceBus
{
    /// <summary>
    /// The Service Bus Management API includes operations for managing Service
    /// Bus queues.
    /// </summary>
    internal partial class QueueOperations : IServiceOperations<ServiceBusManagementClient>, IQueueOperations
    {
        /// <summary>
        /// Initializes a new instance of the QueueOperations class.
        /// </summary>
        /// <param name='client'>
        /// Reference to the service client.
        /// </param>
        internal QueueOperations(ServiceBusManagementClient client)
        {
            this._client = client;
        }
        
        private ServiceBusManagementClient _client;
        
        /// <summary>
        /// Gets a reference to the
        /// Microsoft.WindowsAzure.Management.ServiceBus.ServiceBusManagementClient.
        /// </summary>
        public ServiceBusManagementClient Client
        {
            get { return this._client; }
        }
        
        /// <summary>
        /// Creates a new queue. Once created, this queue's resource manifest
        /// is immutable. This operation is idempotent. Repeating the create
        /// call, after a queue with same name has been created successfully,
        /// will result in a 409 Conflict error message.  (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/jj856295.aspx
        /// for more information)
        /// </summary>
        /// <param name='namespaceName'>
        /// Required. The namespace name.
        /// </param>
        /// <param name='queue'>
        /// Required. The service bus queue.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A response to a request for a particular queue.
        /// </returns>
        public async Task<ServiceBusQueueResponse> CreateAsync(string namespaceName, ServiceBusQueueCreateParameters queue, CancellationToken cancellationToken)
        {
            // Validate
            if (namespaceName == null)
            {
                throw new ArgumentNullException("namespaceName");
            }
            if (queue == null)
            {
                throw new ArgumentNullException("queue");
            }
            if (queue.Name == null)
            {
                throw new ArgumentNullException("queue.Name");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("namespaceName", namespaceName);
                tracingParameters.Add("queue", queue);
                TracingAdapter.Enter(invocationId, this, "CreateAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/services/servicebus/namespaces/";
            url = url + Uri.EscapeDataString(namespaceName);
            url = url + "/queues/";
            url = url + Uri.EscapeDataString(queue.Name);
            url = url + "/";
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Put;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("type", "entry");
                httpRequest.Headers.Add("x-ms-version", "2013-08-01");
                httpRequest.Headers.Add("x-process-at", "ServiceBus");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Serialize Request
                string requestContent = null;
                XDocument requestDoc = new XDocument();
                
                XElement entryElement = new XElement(XName.Get("entry", "http://www.w3.org/2005/Atom"));
                requestDoc.Add(entryElement);
                
                XElement contentElement = new XElement(XName.Get("content", "http://www.w3.org/2005/Atom"));
                entryElement.Add(contentElement);
                
                XAttribute typeAttribute = new XAttribute(XName.Get("type", ""), "");
                typeAttribute.Value = "application/atom+xml;type=entry;charset=utf-8";
                contentElement.Add(typeAttribute);
                
                XElement queueDescriptionElement = new XElement(XName.Get("QueueDescription", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                contentElement.Add(queueDescriptionElement);
                
                if (queue.LockDuration != null)
                {
                    XElement lockDurationElement = new XElement(XName.Get("LockDuration", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                    lockDurationElement.Value = queue.LockDuration;
                    queueDescriptionElement.Add(lockDurationElement);
                }
                
                XElement maxSizeInMegabytesElement = new XElement(XName.Get("MaxSizeInMegabytes", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                maxSizeInMegabytesElement.Value = queue.MaxSizeInMegabytes.ToString();
                queueDescriptionElement.Add(maxSizeInMegabytesElement);
                
                XElement requiresDuplicateDetectionElement = new XElement(XName.Get("RequiresDuplicateDetection", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                requiresDuplicateDetectionElement.Value = queue.RequiresDuplicateDetection.ToString().ToLower();
                queueDescriptionElement.Add(requiresDuplicateDetectionElement);
                
                XElement requiresSessionElement = new XElement(XName.Get("RequiresSession", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                requiresSessionElement.Value = queue.RequiresSession.ToString().ToLower();
                queueDescriptionElement.Add(requiresSessionElement);
                
                if (queue.DefaultMessageTimeToLive != null)
                {
                    XElement defaultMessageTimeToLiveElement = new XElement(XName.Get("DefaultMessageTimeToLive", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                    defaultMessageTimeToLiveElement.Value = queue.DefaultMessageTimeToLive;
                    queueDescriptionElement.Add(defaultMessageTimeToLiveElement);
                }
                
                XElement deadLetteringOnMessageExpirationElement = new XElement(XName.Get("DeadLetteringOnMessageExpiration", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                deadLetteringOnMessageExpirationElement.Value = queue.DeadLetteringOnMessageExpiration.ToString().ToLower();
                queueDescriptionElement.Add(deadLetteringOnMessageExpirationElement);
                
                if (queue.DuplicateDetectionHistoryTimeWindow != null)
                {
                    XElement duplicateDetectionHistoryTimeWindowElement = new XElement(XName.Get("DuplicateDetectionHistoryTimeWindow", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                    duplicateDetectionHistoryTimeWindowElement.Value = queue.DuplicateDetectionHistoryTimeWindow;
                    queueDescriptionElement.Add(duplicateDetectionHistoryTimeWindowElement);
                }
                
                XElement enableBatchedOperationsElement = new XElement(XName.Get("EnableBatchedOperations", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                enableBatchedOperationsElement.Value = queue.EnableBatchedOperations.ToString().ToLower();
                queueDescriptionElement.Add(enableBatchedOperationsElement);
                
                XElement sizeInBytesElement = new XElement(XName.Get("SizeInBytes", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                sizeInBytesElement.Value = queue.SizeInBytes.ToString();
                queueDescriptionElement.Add(sizeInBytesElement);
                
                XElement messageCountElement = new XElement(XName.Get("MessageCount", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                messageCountElement.Value = queue.MessageCount.ToString();
                queueDescriptionElement.Add(messageCountElement);
                
                XElement isAnonymousAccessibleElement = new XElement(XName.Get("IsAnonymousAccessible", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                isAnonymousAccessibleElement.Value = queue.IsAnonymousAccessible.ToString().ToLower();
                queueDescriptionElement.Add(isAnonymousAccessibleElement);
                
                if (queue.AuthorizationRules != null)
                {
                    if (queue.AuthorizationRules is ILazyCollection == false || ((ILazyCollection)queue.AuthorizationRules).IsInitialized)
                    {
                        XElement authorizationRulesSequenceElement = new XElement(XName.Get("AuthorizationRules", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                        foreach (ServiceBusSharedAccessAuthorizationRule authorizationRulesItem in queue.AuthorizationRules)
                        {
                            XElement authorizationRuleElement = new XElement(XName.Get("AuthorizationRule", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                            authorizationRulesSequenceElement.Add(authorizationRuleElement);
                            
                            XAttribute typeAttribute2 = new XAttribute(XName.Get("type", "http://www.w3.org/2001/XMLSchema-instance"), "");
                            typeAttribute2.Value = "SharedAccessAuthorizationRule";
                            authorizationRuleElement.Add(typeAttribute2);
                            
                            if (authorizationRulesItem.ClaimType != null)
                            {
                                XElement claimTypeElement = new XElement(XName.Get("ClaimType", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                claimTypeElement.Value = authorizationRulesItem.ClaimType;
                                authorizationRuleElement.Add(claimTypeElement);
                            }
                            
                            if (authorizationRulesItem.ClaimValue != null)
                            {
                                XElement claimValueElement = new XElement(XName.Get("ClaimValue", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                claimValueElement.Value = authorizationRulesItem.ClaimValue;
                                authorizationRuleElement.Add(claimValueElement);
                            }
                            
                            if (authorizationRulesItem.Rights != null)
                            {
                                if (authorizationRulesItem.Rights is ILazyCollection == false || ((ILazyCollection)authorizationRulesItem.Rights).IsInitialized)
                                {
                                    XElement rightsSequenceElement = new XElement(XName.Get("Rights", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    foreach (AccessRight rightsItem in authorizationRulesItem.Rights)
                                    {
                                        XElement rightsItemElement = new XElement(XName.Get("AccessRights", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                        rightsItemElement.Value = rightsItem.ToString();
                                        rightsSequenceElement.Add(rightsItemElement);
                                    }
                                    authorizationRuleElement.Add(rightsSequenceElement);
                                }
                            }
                            
                            XElement createdTimeElement = new XElement(XName.Get("CreatedTime", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                            createdTimeElement.Value = authorizationRulesItem.CreatedTime.ToString();
                            authorizationRuleElement.Add(createdTimeElement);
                            
                            if (authorizationRulesItem.KeyName != null)
                            {
                                XElement keyNameElement = new XElement(XName.Get("KeyName", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                keyNameElement.Value = authorizationRulesItem.KeyName;
                                authorizationRuleElement.Add(keyNameElement);
                            }
                            
                            XElement modifiedTimeElement = new XElement(XName.Get("ModifiedTime", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                            modifiedTimeElement.Value = authorizationRulesItem.ModifiedTime.ToString();
                            authorizationRuleElement.Add(modifiedTimeElement);
                            
                            if (authorizationRulesItem.PrimaryKey != null)
                            {
                                XElement primaryKeyElement = new XElement(XName.Get("PrimaryKey", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                primaryKeyElement.Value = authorizationRulesItem.PrimaryKey;
                                authorizationRuleElement.Add(primaryKeyElement);
                            }
                            
                            if (authorizationRulesItem.SecondaryKey != null)
                            {
                                XElement secondaryKeyElement = new XElement(XName.Get("SecondaryKey", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                secondaryKeyElement.Value = authorizationRulesItem.SecondaryKey;
                                authorizationRuleElement.Add(secondaryKeyElement);
                            }
                        }
                        queueDescriptionElement.Add(authorizationRulesSequenceElement);
                    }
                }
                
                if (queue.Status != null)
                {
                    XElement statusElement = new XElement(XName.Get("Status", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                    statusElement.Value = queue.Status;
                    queueDescriptionElement.Add(statusElement);
                }
                
                XElement supportOrderingElement = new XElement(XName.Get("SupportOrdering", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                supportOrderingElement.Value = queue.SupportOrdering.ToString().ToLower();
                queueDescriptionElement.Add(supportOrderingElement);
                
                if (queue.CountDetails != null)
                {
                    XElement countDetailsElement = new XElement(XName.Get("CountDetails", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                    queueDescriptionElement.Add(countDetailsElement);
                    
                    XElement activeMessageCountElement = new XElement(XName.Get("ActiveMessageCount", "http://schemas.microsoft.com/netservices/2011/06/servicebus"));
                    activeMessageCountElement.Value = queue.CountDetails.ActiveMessageCount.ToString();
                    countDetailsElement.Add(activeMessageCountElement);
                    
                    XElement deadLetterMessageCountElement = new XElement(XName.Get("DeadLetterMessageCount", "http://schemas.microsoft.com/netservices/2011/06/servicebus"));
                    deadLetterMessageCountElement.Value = queue.CountDetails.DeadLetterMessageCount.ToString();
                    countDetailsElement.Add(deadLetterMessageCountElement);
                    
                    XElement scheduledMessageCountElement = new XElement(XName.Get("ScheduledMessageCount", "http://schemas.microsoft.com/netservices/2011/06/servicebus"));
                    scheduledMessageCountElement.Value = queue.CountDetails.ScheduledMessageCount.ToString();
                    countDetailsElement.Add(scheduledMessageCountElement);
                    
                    XElement transferDeadLetterMessageCountElement = new XElement(XName.Get("TransferDeadLetterMessageCount", "http://schemas.microsoft.com/netservices/2011/06/servicebus"));
                    transferDeadLetterMessageCountElement.Value = queue.CountDetails.TransferDeadLetterMessageCount.ToString();
                    countDetailsElement.Add(transferDeadLetterMessageCountElement);
                    
                    XElement transferMessageCountElement = new XElement(XName.Get("TransferMessageCount", "http://schemas.microsoft.com/netservices/2011/06/servicebus"));
                    transferMessageCountElement.Value = queue.CountDetails.TransferMessageCount.ToString();
                    countDetailsElement.Add(transferMessageCountElement);
                }
                
                if (queue.AutoDeleteOnIdle != null)
                {
                    XElement autoDeleteOnIdleElement = new XElement(XName.Get("AutoDeleteOnIdle", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                    autoDeleteOnIdleElement.Value = queue.AutoDeleteOnIdle;
                    queueDescriptionElement.Add(autoDeleteOnIdleElement);
                }
                
                if (queue.EntityAvailabilityStatus != null)
                {
                    XElement entityAvailabilityStatusElement = new XElement(XName.Get("EntityAvailabilityStatus", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                    entityAvailabilityStatusElement.Value = queue.EntityAvailabilityStatus;
                    queueDescriptionElement.Add(entityAvailabilityStatusElement);
                }
                
                requestContent = requestDoc.ToString();
                httpRequest.Content = new StringContent(requestContent, Encoding.UTF8);
                httpRequest.Content.Headers.ContentType = MediaTypeHeaderValue.Parse("application/atom+xml");
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.Created)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, requestContent, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    ServiceBusQueueResponse result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.Created)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new ServiceBusQueueResponse();
                        XDocument responseDoc = XDocument.Parse(responseContent);
                        
                        XElement entryElement2 = responseDoc.Element(XName.Get("entry", "http://www.w3.org/2005/Atom"));
                        if (entryElement2 != null)
                        {
                            XElement titleElement = entryElement2.Element(XName.Get("title", "http://www.w3.org/2005/Atom"));
                            if (titleElement != null)
                            {
                            }
                            
                            XElement contentElement2 = entryElement2.Element(XName.Get("content", "http://www.w3.org/2005/Atom"));
                            if (contentElement2 != null)
                            {
                                XElement queueDescriptionElement2 = contentElement2.Element(XName.Get("QueueDescription", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (queueDescriptionElement2 != null)
                                {
                                    ServiceBusQueue queueDescriptionInstance = new ServiceBusQueue();
                                    result.Queue = queueDescriptionInstance;
                                    
                                    XElement lockDurationElement2 = queueDescriptionElement2.Element(XName.Get("LockDuration", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (lockDurationElement2 != null)
                                    {
                                        string lockDurationInstance = lockDurationElement2.Value;
                                        queueDescriptionInstance.LockDuration = lockDurationInstance;
                                    }
                                    
                                    XElement maxSizeInMegabytesElement2 = queueDescriptionElement2.Element(XName.Get("MaxSizeInMegabytes", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (maxSizeInMegabytesElement2 != null)
                                    {
                                        int maxSizeInMegabytesInstance = int.Parse(maxSizeInMegabytesElement2.Value, CultureInfo.InvariantCulture);
                                        queueDescriptionInstance.MaxSizeInMegabytes = maxSizeInMegabytesInstance;
                                    }
                                    
                                    XElement requiresDuplicateDetectionElement2 = queueDescriptionElement2.Element(XName.Get("RequiresDuplicateDetection", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (requiresDuplicateDetectionElement2 != null)
                                    {
                                        bool requiresDuplicateDetectionInstance = bool.Parse(requiresDuplicateDetectionElement2.Value);
                                        queueDescriptionInstance.RequiresDuplicateDetection = requiresDuplicateDetectionInstance;
                                    }
                                    
                                    XElement requiresSessionElement2 = queueDescriptionElement2.Element(XName.Get("RequiresSession", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (requiresSessionElement2 != null)
                                    {
                                        bool requiresSessionInstance = bool.Parse(requiresSessionElement2.Value);
                                        queueDescriptionInstance.RequiresSession = requiresSessionInstance;
                                    }
                                    
                                    XElement defaultMessageTimeToLiveElement2 = queueDescriptionElement2.Element(XName.Get("DefaultMessageTimeToLive", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (defaultMessageTimeToLiveElement2 != null)
                                    {
                                        string defaultMessageTimeToLiveInstance = defaultMessageTimeToLiveElement2.Value;
                                        queueDescriptionInstance.DefaultMessageTimeToLive = defaultMessageTimeToLiveInstance;
                                    }
                                    
                                    XElement deadLetteringOnMessageExpirationElement2 = queueDescriptionElement2.Element(XName.Get("DeadLetteringOnMessageExpiration", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (deadLetteringOnMessageExpirationElement2 != null)
                                    {
                                        bool deadLetteringOnMessageExpirationInstance = bool.Parse(deadLetteringOnMessageExpirationElement2.Value);
                                        queueDescriptionInstance.DeadLetteringOnMessageExpiration = deadLetteringOnMessageExpirationInstance;
                                    }
                                    
                                    XElement duplicateDetectionHistoryTimeWindowElement2 = queueDescriptionElement2.Element(XName.Get("DuplicateDetectionHistoryTimeWindow", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (duplicateDetectionHistoryTimeWindowElement2 != null)
                                    {
                                        string duplicateDetectionHistoryTimeWindowInstance = duplicateDetectionHistoryTimeWindowElement2.Value;
                                        queueDescriptionInstance.DuplicateDetectionHistoryTimeWindow = duplicateDetectionHistoryTimeWindowInstance;
                                    }
                                    
                                    XElement maxDeliveryCountElement = queueDescriptionElement2.Element(XName.Get("MaxDeliveryCount", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (maxDeliveryCountElement != null)
                                    {
                                        int maxDeliveryCountInstance = int.Parse(maxDeliveryCountElement.Value, CultureInfo.InvariantCulture);
                                        queueDescriptionInstance.MaxDeliveryCount = maxDeliveryCountInstance;
                                    }
                                    
                                    XElement enableBatchedOperationsElement2 = queueDescriptionElement2.Element(XName.Get("EnableBatchedOperations", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (enableBatchedOperationsElement2 != null)
                                    {
                                        bool enableBatchedOperationsInstance = bool.Parse(enableBatchedOperationsElement2.Value);
                                        queueDescriptionInstance.EnableBatchedOperations = enableBatchedOperationsInstance;
                                    }
                                    
                                    XElement sizeInBytesElement2 = queueDescriptionElement2.Element(XName.Get("SizeInBytes", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (sizeInBytesElement2 != null)
                                    {
                                        int sizeInBytesInstance = int.Parse(sizeInBytesElement2.Value, CultureInfo.InvariantCulture);
                                        queueDescriptionInstance.SizeInBytes = sizeInBytesInstance;
                                    }
                                    
                                    XElement messageCountElement2 = queueDescriptionElement2.Element(XName.Get("MessageCount", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (messageCountElement2 != null)
                                    {
                                        int messageCountInstance = int.Parse(messageCountElement2.Value, CultureInfo.InvariantCulture);
                                        queueDescriptionInstance.MessageCount = messageCountInstance;
                                    }
                                    
                                    XElement isAnonymousAccessibleElement2 = queueDescriptionElement2.Element(XName.Get("IsAnonymousAccessible", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (isAnonymousAccessibleElement2 != null)
                                    {
                                        bool isAnonymousAccessibleInstance = bool.Parse(isAnonymousAccessibleElement2.Value);
                                        queueDescriptionInstance.IsAnonymousAccessible = isAnonymousAccessibleInstance;
                                    }
                                    
                                    XElement authorizationRulesSequenceElement2 = queueDescriptionElement2.Element(XName.Get("AuthorizationRules", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (authorizationRulesSequenceElement2 != null)
                                    {
                                        foreach (XElement authorizationRulesElement in authorizationRulesSequenceElement2.Elements(XName.Get("AuthorizationRule", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect")))
                                        {
                                            ServiceBusSharedAccessAuthorizationRule authorizationRuleInstance = new ServiceBusSharedAccessAuthorizationRule();
                                            queueDescriptionInstance.AuthorizationRules.Add(authorizationRuleInstance);
                                            
                                            XElement claimTypeElement2 = authorizationRulesElement.Element(XName.Get("ClaimType", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                            if (claimTypeElement2 != null)
                                            {
                                                string claimTypeInstance = claimTypeElement2.Value;
                                                authorizationRuleInstance.ClaimType = claimTypeInstance;
                                            }
                                            
                                            XElement claimValueElement2 = authorizationRulesElement.Element(XName.Get("ClaimValue", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                            if (claimValueElement2 != null)
                                            {
                                                string claimValueInstance = claimValueElement2.Value;
                                                authorizationRuleInstance.ClaimValue = claimValueInstance;
                                            }
                                            
                                            XElement rightsSequenceElement2 = authorizationRulesElement.Element(XName.Get("Rights", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                            if (rightsSequenceElement2 != null)
                                            {
                                                foreach (XElement rightsElement in rightsSequenceElement2.Elements(XName.Get("AccessRights", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect")))
                                                {
                                                    authorizationRuleInstance.Rights.Add(((AccessRight)Enum.Parse(typeof(AccessRight), rightsElement.Value, true)));
                                                }
                                            }
                                            
                                            XElement createdTimeElement2 = authorizationRulesElement.Element(XName.Get("CreatedTime", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                            if (createdTimeElement2 != null)
                                            {
                                                DateTime createdTimeInstance = DateTime.Parse(createdTimeElement2.Value, CultureInfo.InvariantCulture);
                                                authorizationRuleInstance.CreatedTime = createdTimeInstance;
                                            }
                                            
                                            XElement keyNameElement2 = authorizationRulesElement.Element(XName.Get("KeyName", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                            if (keyNameElement2 != null)
                                            {
                                                string keyNameInstance = keyNameElement2.Value;
                                                authorizationRuleInstance.KeyName = keyNameInstance;
                                            }
                                            
                                            XElement modifiedTimeElement2 = authorizationRulesElement.Element(XName.Get("ModifiedTime", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                            if (modifiedTimeElement2 != null)
                                            {
                                                DateTime modifiedTimeInstance = DateTime.Parse(modifiedTimeElement2.Value, CultureInfo.InvariantCulture);
                                                authorizationRuleInstance.ModifiedTime = modifiedTimeInstance;
                                            }
                                            
                                            XElement primaryKeyElement2 = authorizationRulesElement.Element(XName.Get("PrimaryKey", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                            if (primaryKeyElement2 != null)
                                            {
                                                string primaryKeyInstance = primaryKeyElement2.Value;
                                                authorizationRuleInstance.PrimaryKey = primaryKeyInstance;
                                            }
                                            
                                            XElement secondaryKeyElement2 = authorizationRulesElement.Element(XName.Get("SecondaryKey", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                            if (secondaryKeyElement2 != null)
                                            {
                                                string secondaryKeyInstance = secondaryKeyElement2.Value;
                                                authorizationRuleInstance.SecondaryKey = secondaryKeyInstance;
                                            }
                                        }
                                    }
                                    
                                    XElement statusElement2 = queueDescriptionElement2.Element(XName.Get("Status", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (statusElement2 != null)
                                    {
                                        string statusInstance = statusElement2.Value;
                                        queueDescriptionInstance.Status = statusInstance;
                                    }
                                    
                                    XElement createdAtElement = queueDescriptionElement2.Element(XName.Get("CreatedAt", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (createdAtElement != null)
                                    {
                                        DateTime createdAtInstance = DateTime.Parse(createdAtElement.Value, CultureInfo.InvariantCulture);
                                        queueDescriptionInstance.CreatedAt = createdAtInstance;
                                    }
                                    
                                    XElement updatedAtElement = queueDescriptionElement2.Element(XName.Get("UpdatedAt", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (updatedAtElement != null)
                                    {
                                        DateTime updatedAtInstance = DateTime.Parse(updatedAtElement.Value, CultureInfo.InvariantCulture);
                                        queueDescriptionInstance.UpdatedAt = updatedAtInstance;
                                    }
                                    
                                    XElement accessedAtElement = queueDescriptionElement2.Element(XName.Get("AccessedAt", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (accessedAtElement != null)
                                    {
                                        DateTime accessedAtInstance = DateTime.Parse(accessedAtElement.Value, CultureInfo.InvariantCulture);
                                        queueDescriptionInstance.AccessedAt = accessedAtInstance;
                                    }
                                    
                                    XElement supportOrderingElement2 = queueDescriptionElement2.Element(XName.Get("SupportOrdering", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (supportOrderingElement2 != null)
                                    {
                                        bool supportOrderingInstance = bool.Parse(supportOrderingElement2.Value);
                                        queueDescriptionInstance.SupportOrdering = supportOrderingInstance;
                                    }
                                    
                                    XElement countDetailsElement2 = queueDescriptionElement2.Element(XName.Get("CountDetails", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (countDetailsElement2 != null)
                                    {
                                        CountDetails countDetailsInstance = new CountDetails();
                                        queueDescriptionInstance.CountDetails = countDetailsInstance;
                                        
                                        XElement activeMessageCountElement2 = countDetailsElement2.Element(XName.Get("ActiveMessageCount", "http://schemas.microsoft.com/netservices/2011/06/servicebus"));
                                        if (activeMessageCountElement2 != null)
                                        {
                                            int activeMessageCountInstance = int.Parse(activeMessageCountElement2.Value, CultureInfo.InvariantCulture);
                                            countDetailsInstance.ActiveMessageCount = activeMessageCountInstance;
                                        }
                                        
                                        XElement deadLetterMessageCountElement2 = countDetailsElement2.Element(XName.Get("DeadLetterMessageCount", "http://schemas.microsoft.com/netservices/2011/06/servicebus"));
                                        if (deadLetterMessageCountElement2 != null)
                                        {
                                            int deadLetterMessageCountInstance = int.Parse(deadLetterMessageCountElement2.Value, CultureInfo.InvariantCulture);
                                            countDetailsInstance.DeadLetterMessageCount = deadLetterMessageCountInstance;
                                        }
                                        
                                        XElement scheduledMessageCountElement2 = countDetailsElement2.Element(XName.Get("ScheduledMessageCount", "http://schemas.microsoft.com/netservices/2011/06/servicebus"));
                                        if (scheduledMessageCountElement2 != null)
                                        {
                                            int scheduledMessageCountInstance = int.Parse(scheduledMessageCountElement2.Value, CultureInfo.InvariantCulture);
                                            countDetailsInstance.ScheduledMessageCount = scheduledMessageCountInstance;
                                        }
                                        
                                        XElement transferDeadLetterMessageCountElement2 = countDetailsElement2.Element(XName.Get("TransferDeadLetterMessageCount", "http://schemas.microsoft.com/netservices/2011/06/servicebus"));
                                        if (transferDeadLetterMessageCountElement2 != null)
                                        {
                                            int transferDeadLetterMessageCountInstance = int.Parse(transferDeadLetterMessageCountElement2.Value, CultureInfo.InvariantCulture);
                                            countDetailsInstance.TransferDeadLetterMessageCount = transferDeadLetterMessageCountInstance;
                                        }
                                        
                                        XElement transferMessageCountElement2 = countDetailsElement2.Element(XName.Get("TransferMessageCount", "http://schemas.microsoft.com/netservices/2011/06/servicebus"));
                                        if (transferMessageCountElement2 != null)
                                        {
                                            int transferMessageCountInstance = int.Parse(transferMessageCountElement2.Value, CultureInfo.InvariantCulture);
                                            countDetailsInstance.TransferMessageCount = transferMessageCountInstance;
                                        }
                                    }
                                    
                                    XElement autoDeleteOnIdleElement2 = queueDescriptionElement2.Element(XName.Get("AutoDeleteOnIdle", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (autoDeleteOnIdleElement2 != null)
                                    {
                                        string autoDeleteOnIdleInstance = autoDeleteOnIdleElement2.Value;
                                        queueDescriptionInstance.AutoDeleteOnIdle = autoDeleteOnIdleInstance;
                                    }
                                    
                                    XElement entityAvailabilityStatusElement2 = queueDescriptionElement2.Element(XName.Get("EntityAvailabilityStatus", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (entityAvailabilityStatusElement2 != null)
                                    {
                                        string entityAvailabilityStatusInstance = entityAvailabilityStatusElement2.Value;
                                        queueDescriptionInstance.EntityAvailabilityStatus = entityAvailabilityStatusInstance;
                                    }
                                }
                            }
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Deletes an existing queue. This operation will also remove all
        /// associated state including messages in the queue.  (see
        /// http://msdn.microsoft.com/en-us/library/hh780747.aspx for more
        /// information)
        /// </summary>
        /// <param name='namespaceName'>
        /// Required. The namespace name.
        /// </param>
        /// <param name='queueName'>
        /// Required. The queue name.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A standard service response including an HTTP status code and
        /// request ID.
        /// </returns>
        public async Task<AzureOperationResponse> DeleteAsync(string namespaceName, string queueName, CancellationToken cancellationToken)
        {
            // Validate
            if (namespaceName == null)
            {
                throw new ArgumentNullException("namespaceName");
            }
            if (queueName == null)
            {
                throw new ArgumentNullException("queueName");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("namespaceName", namespaceName);
                tracingParameters.Add("queueName", queueName);
                TracingAdapter.Enter(invocationId, this, "DeleteAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/services/servicebus/namespaces/";
            url = url + Uri.EscapeDataString(namespaceName);
            url = url + "/Queues/";
            url = url + Uri.EscapeDataString(queueName);
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Delete;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("x-ms-version", "2013-08-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    AzureOperationResponse result = null;
                    // Deserialize Response
                    result = new AzureOperationResponse();
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// The queue description is an XML AtomPub document that defines the
        /// desired semantics for a subscription. The queue description
        /// contains the following properties. For more information, see the
        /// QueueDescription Properties topic.  (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/hh780773.aspx
        /// for more information)
        /// </summary>
        /// <param name='namespaceName'>
        /// Required. The namespace name.
        /// </param>
        /// <param name='queueName'>
        /// Required. The queue name.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A response to a request for a particular queue.
        /// </returns>
        public async Task<ServiceBusQueueResponse> GetAsync(string namespaceName, string queueName, CancellationToken cancellationToken)
        {
            // Validate
            if (namespaceName == null)
            {
                throw new ArgumentNullException("namespaceName");
            }
            if (queueName == null)
            {
                throw new ArgumentNullException("queueName");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("namespaceName", namespaceName);
                tracingParameters.Add("queueName", queueName);
                TracingAdapter.Enter(invocationId, this, "GetAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/services/servicebus/namespaces/";
            url = url + Uri.EscapeDataString(namespaceName);
            url = url + "/Queues/";
            url = url + Uri.EscapeDataString(queueName);
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("x-ms-version", "2013-08-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    ServiceBusQueueResponse result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new ServiceBusQueueResponse();
                        XDocument responseDoc = XDocument.Parse(responseContent);
                        
                        XElement entryElement = responseDoc.Element(XName.Get("entry", "http://www.w3.org/2005/Atom"));
                        if (entryElement != null)
                        {
                            XElement titleElement = entryElement.Element(XName.Get("title", "http://www.w3.org/2005/Atom"));
                            if (titleElement != null)
                            {
                            }
                            
                            XElement contentElement = entryElement.Element(XName.Get("content", "http://www.w3.org/2005/Atom"));
                            if (contentElement != null)
                            {
                                XElement queueDescriptionElement = contentElement.Element(XName.Get("QueueDescription", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (queueDescriptionElement != null)
                                {
                                    ServiceBusQueue queueDescriptionInstance = new ServiceBusQueue();
                                    result.Queue = queueDescriptionInstance;
                                    
                                    XElement lockDurationElement = queueDescriptionElement.Element(XName.Get("LockDuration", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (lockDurationElement != null)
                                    {
                                        string lockDurationInstance = lockDurationElement.Value;
                                        queueDescriptionInstance.LockDuration = lockDurationInstance;
                                    }
                                    
                                    XElement maxSizeInMegabytesElement = queueDescriptionElement.Element(XName.Get("MaxSizeInMegabytes", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (maxSizeInMegabytesElement != null)
                                    {
                                        int maxSizeInMegabytesInstance = int.Parse(maxSizeInMegabytesElement.Value, CultureInfo.InvariantCulture);
                                        queueDescriptionInstance.MaxSizeInMegabytes = maxSizeInMegabytesInstance;
                                    }
                                    
                                    XElement requiresDuplicateDetectionElement = queueDescriptionElement.Element(XName.Get("RequiresDuplicateDetection", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (requiresDuplicateDetectionElement != null)
                                    {
                                        bool requiresDuplicateDetectionInstance = bool.Parse(requiresDuplicateDetectionElement.Value);
                                        queueDescriptionInstance.RequiresDuplicateDetection = requiresDuplicateDetectionInstance;
                                    }
                                    
                                    XElement requiresSessionElement = queueDescriptionElement.Element(XName.Get("RequiresSession", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (requiresSessionElement != null)
                                    {
                                        bool requiresSessionInstance = bool.Parse(requiresSessionElement.Value);
                                        queueDescriptionInstance.RequiresSession = requiresSessionInstance;
                                    }
                                    
                                    XElement defaultMessageTimeToLiveElement = queueDescriptionElement.Element(XName.Get("DefaultMessageTimeToLive", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (defaultMessageTimeToLiveElement != null)
                                    {
                                        string defaultMessageTimeToLiveInstance = defaultMessageTimeToLiveElement.Value;
                                        queueDescriptionInstance.DefaultMessageTimeToLive = defaultMessageTimeToLiveInstance;
                                    }
                                    
                                    XElement deadLetteringOnMessageExpirationElement = queueDescriptionElement.Element(XName.Get("DeadLetteringOnMessageExpiration", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (deadLetteringOnMessageExpirationElement != null)
                                    {
                                        bool deadLetteringOnMessageExpirationInstance = bool.Parse(deadLetteringOnMessageExpirationElement.Value);
                                        queueDescriptionInstance.DeadLetteringOnMessageExpiration = deadLetteringOnMessageExpirationInstance;
                                    }
                                    
                                    XElement duplicateDetectionHistoryTimeWindowElement = queueDescriptionElement.Element(XName.Get("DuplicateDetectionHistoryTimeWindow", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (duplicateDetectionHistoryTimeWindowElement != null)
                                    {
                                        string duplicateDetectionHistoryTimeWindowInstance = duplicateDetectionHistoryTimeWindowElement.Value;
                                        queueDescriptionInstance.DuplicateDetectionHistoryTimeWindow = duplicateDetectionHistoryTimeWindowInstance;
                                    }
                                    
                                    XElement maxDeliveryCountElement = queueDescriptionElement.Element(XName.Get("MaxDeliveryCount", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (maxDeliveryCountElement != null)
                                    {
                                        int maxDeliveryCountInstance = int.Parse(maxDeliveryCountElement.Value, CultureInfo.InvariantCulture);
                                        queueDescriptionInstance.MaxDeliveryCount = maxDeliveryCountInstance;
                                    }
                                    
                                    XElement enableBatchedOperationsElement = queueDescriptionElement.Element(XName.Get("EnableBatchedOperations", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (enableBatchedOperationsElement != null)
                                    {
                                        bool enableBatchedOperationsInstance = bool.Parse(enableBatchedOperationsElement.Value);
                                        queueDescriptionInstance.EnableBatchedOperations = enableBatchedOperationsInstance;
                                    }
                                    
                                    XElement sizeInBytesElement = queueDescriptionElement.Element(XName.Get("SizeInBytes", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (sizeInBytesElement != null)
                                    {
                                        int sizeInBytesInstance = int.Parse(sizeInBytesElement.Value, CultureInfo.InvariantCulture);
                                        queueDescriptionInstance.SizeInBytes = sizeInBytesInstance;
                                    }
                                    
                                    XElement messageCountElement = queueDescriptionElement.Element(XName.Get("MessageCount", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (messageCountElement != null)
                                    {
                                        int messageCountInstance = int.Parse(messageCountElement.Value, CultureInfo.InvariantCulture);
                                        queueDescriptionInstance.MessageCount = messageCountInstance;
                                    }
                                    
                                    XElement isAnonymousAccessibleElement = queueDescriptionElement.Element(XName.Get("IsAnonymousAccessible", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (isAnonymousAccessibleElement != null)
                                    {
                                        bool isAnonymousAccessibleInstance = bool.Parse(isAnonymousAccessibleElement.Value);
                                        queueDescriptionInstance.IsAnonymousAccessible = isAnonymousAccessibleInstance;
                                    }
                                    
                                    XElement authorizationRulesSequenceElement = queueDescriptionElement.Element(XName.Get("AuthorizationRules", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (authorizationRulesSequenceElement != null)
                                    {
                                        foreach (XElement authorizationRulesElement in authorizationRulesSequenceElement.Elements(XName.Get("AuthorizationRule", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect")))
                                        {
                                            ServiceBusSharedAccessAuthorizationRule authorizationRuleInstance = new ServiceBusSharedAccessAuthorizationRule();
                                            queueDescriptionInstance.AuthorizationRules.Add(authorizationRuleInstance);
                                            
                                            XElement claimTypeElement = authorizationRulesElement.Element(XName.Get("ClaimType", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                            if (claimTypeElement != null)
                                            {
                                                string claimTypeInstance = claimTypeElement.Value;
                                                authorizationRuleInstance.ClaimType = claimTypeInstance;
                                            }
                                            
                                            XElement claimValueElement = authorizationRulesElement.Element(XName.Get("ClaimValue", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                            if (claimValueElement != null)
                                            {
                                                string claimValueInstance = claimValueElement.Value;
                                                authorizationRuleInstance.ClaimValue = claimValueInstance;
                                            }
                                            
                                            XElement rightsSequenceElement = authorizationRulesElement.Element(XName.Get("Rights", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                            if (rightsSequenceElement != null)
                                            {
                                                foreach (XElement rightsElement in rightsSequenceElement.Elements(XName.Get("AccessRights", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect")))
                                                {
                                                    authorizationRuleInstance.Rights.Add(((AccessRight)Enum.Parse(typeof(AccessRight), rightsElement.Value, true)));
                                                }
                                            }
                                            
                                            XElement createdTimeElement = authorizationRulesElement.Element(XName.Get("CreatedTime", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                            if (createdTimeElement != null)
                                            {
                                                DateTime createdTimeInstance = DateTime.Parse(createdTimeElement.Value, CultureInfo.InvariantCulture);
                                                authorizationRuleInstance.CreatedTime = createdTimeInstance;
                                            }
                                            
                                            XElement keyNameElement = authorizationRulesElement.Element(XName.Get("KeyName", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                            if (keyNameElement != null)
                                            {
                                                string keyNameInstance = keyNameElement.Value;
                                                authorizationRuleInstance.KeyName = keyNameInstance;
                                            }
                                            
                                            XElement modifiedTimeElement = authorizationRulesElement.Element(XName.Get("ModifiedTime", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                            if (modifiedTimeElement != null)
                                            {
                                                DateTime modifiedTimeInstance = DateTime.Parse(modifiedTimeElement.Value, CultureInfo.InvariantCulture);
                                                authorizationRuleInstance.ModifiedTime = modifiedTimeInstance;
                                            }
                                            
                                            XElement primaryKeyElement = authorizationRulesElement.Element(XName.Get("PrimaryKey", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                            if (primaryKeyElement != null)
                                            {
                                                string primaryKeyInstance = primaryKeyElement.Value;
                                                authorizationRuleInstance.PrimaryKey = primaryKeyInstance;
                                            }
                                            
                                            XElement secondaryKeyElement = authorizationRulesElement.Element(XName.Get("SecondaryKey", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                            if (secondaryKeyElement != null)
                                            {
                                                string secondaryKeyInstance = secondaryKeyElement.Value;
                                                authorizationRuleInstance.SecondaryKey = secondaryKeyInstance;
                                            }
                                        }
                                    }
                                    
                                    XElement statusElement = queueDescriptionElement.Element(XName.Get("Status", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (statusElement != null)
                                    {
                                        string statusInstance = statusElement.Value;
                                        queueDescriptionInstance.Status = statusInstance;
                                    }
                                    
                                    XElement createdAtElement = queueDescriptionElement.Element(XName.Get("CreatedAt", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (createdAtElement != null)
                                    {
                                        DateTime createdAtInstance = DateTime.Parse(createdAtElement.Value, CultureInfo.InvariantCulture);
                                        queueDescriptionInstance.CreatedAt = createdAtInstance;
                                    }
                                    
                                    XElement updatedAtElement = queueDescriptionElement.Element(XName.Get("UpdatedAt", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (updatedAtElement != null)
                                    {
                                        DateTime updatedAtInstance = DateTime.Parse(updatedAtElement.Value, CultureInfo.InvariantCulture);
                                        queueDescriptionInstance.UpdatedAt = updatedAtInstance;
                                    }
                                    
                                    XElement accessedAtElement = queueDescriptionElement.Element(XName.Get("AccessedAt", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (accessedAtElement != null)
                                    {
                                        DateTime accessedAtInstance = DateTime.Parse(accessedAtElement.Value, CultureInfo.InvariantCulture);
                                        queueDescriptionInstance.AccessedAt = accessedAtInstance;
                                    }
                                    
                                    XElement supportOrderingElement = queueDescriptionElement.Element(XName.Get("SupportOrdering", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (supportOrderingElement != null)
                                    {
                                        bool supportOrderingInstance = bool.Parse(supportOrderingElement.Value);
                                        queueDescriptionInstance.SupportOrdering = supportOrderingInstance;
                                    }
                                    
                                    XElement countDetailsElement = queueDescriptionElement.Element(XName.Get("CountDetails", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (countDetailsElement != null)
                                    {
                                        CountDetails countDetailsInstance = new CountDetails();
                                        queueDescriptionInstance.CountDetails = countDetailsInstance;
                                        
                                        XElement activeMessageCountElement = countDetailsElement.Element(XName.Get("ActiveMessageCount", "http://schemas.microsoft.com/netservices/2011/06/servicebus"));
                                        if (activeMessageCountElement != null)
                                        {
                                            int activeMessageCountInstance = int.Parse(activeMessageCountElement.Value, CultureInfo.InvariantCulture);
                                            countDetailsInstance.ActiveMessageCount = activeMessageCountInstance;
                                        }
                                        
                                        XElement deadLetterMessageCountElement = countDetailsElement.Element(XName.Get("DeadLetterMessageCount", "http://schemas.microsoft.com/netservices/2011/06/servicebus"));
                                        if (deadLetterMessageCountElement != null)
                                        {
                                            int deadLetterMessageCountInstance = int.Parse(deadLetterMessageCountElement.Value, CultureInfo.InvariantCulture);
                                            countDetailsInstance.DeadLetterMessageCount = deadLetterMessageCountInstance;
                                        }
                                        
                                        XElement scheduledMessageCountElement = countDetailsElement.Element(XName.Get("ScheduledMessageCount", "http://schemas.microsoft.com/netservices/2011/06/servicebus"));
                                        if (scheduledMessageCountElement != null)
                                        {
                                            int scheduledMessageCountInstance = int.Parse(scheduledMessageCountElement.Value, CultureInfo.InvariantCulture);
                                            countDetailsInstance.ScheduledMessageCount = scheduledMessageCountInstance;
                                        }
                                        
                                        XElement transferDeadLetterMessageCountElement = countDetailsElement.Element(XName.Get("TransferDeadLetterMessageCount", "http://schemas.microsoft.com/netservices/2011/06/servicebus"));
                                        if (transferDeadLetterMessageCountElement != null)
                                        {
                                            int transferDeadLetterMessageCountInstance = int.Parse(transferDeadLetterMessageCountElement.Value, CultureInfo.InvariantCulture);
                                            countDetailsInstance.TransferDeadLetterMessageCount = transferDeadLetterMessageCountInstance;
                                        }
                                        
                                        XElement transferMessageCountElement = countDetailsElement.Element(XName.Get("TransferMessageCount", "http://schemas.microsoft.com/netservices/2011/06/servicebus"));
                                        if (transferMessageCountElement != null)
                                        {
                                            int transferMessageCountInstance = int.Parse(transferMessageCountElement.Value, CultureInfo.InvariantCulture);
                                            countDetailsInstance.TransferMessageCount = transferMessageCountInstance;
                                        }
                                    }
                                    
                                    XElement autoDeleteOnIdleElement = queueDescriptionElement.Element(XName.Get("AutoDeleteOnIdle", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (autoDeleteOnIdleElement != null)
                                    {
                                        string autoDeleteOnIdleInstance = autoDeleteOnIdleElement.Value;
                                        queueDescriptionInstance.AutoDeleteOnIdle = autoDeleteOnIdleInstance;
                                    }
                                    
                                    XElement entityAvailabilityStatusElement = queueDescriptionElement.Element(XName.Get("EntityAvailabilityStatus", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (entityAvailabilityStatusElement != null)
                                    {
                                        string entityAvailabilityStatusInstance = entityAvailabilityStatusElement.Value;
                                        queueDescriptionInstance.EntityAvailabilityStatus = entityAvailabilityStatusInstance;
                                    }
                                }
                            }
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Gets the set of connection strings for a queue.
        /// </summary>
        /// <param name='namespaceName'>
        /// Required. The namespace name.
        /// </param>
        /// <param name='queueName'>
        /// Required. The queue name.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// The set of connection details for a service bus entity.
        /// </returns>
        public async Task<ServiceBusConnectionDetailsResponse> GetConnectionDetailsAsync(string namespaceName, string queueName, CancellationToken cancellationToken)
        {
            // Validate
            if (namespaceName == null)
            {
                throw new ArgumentNullException("namespaceName");
            }
            if (queueName == null)
            {
                throw new ArgumentNullException("queueName");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("namespaceName", namespaceName);
                tracingParameters.Add("queueName", queueName);
                TracingAdapter.Enter(invocationId, this, "GetConnectionDetailsAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/services/servicebus/namespaces/";
            url = url + Uri.EscapeDataString(namespaceName);
            url = url + "/Queues/";
            url = url + Uri.EscapeDataString(queueName);
            url = url + "/ConnectionDetails";
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("x-ms-version", "2013-08-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    ServiceBusConnectionDetailsResponse result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new ServiceBusConnectionDetailsResponse();
                        XDocument responseDoc = XDocument.Parse(responseContent);
                        
                        XElement feedElement = responseDoc.Element(XName.Get("feed", "http://www.w3.org/2005/Atom"));
                        if (feedElement != null)
                        {
                            if (feedElement != null)
                            {
                                foreach (XElement entriesElement in feedElement.Elements(XName.Get("entry", "http://www.w3.org/2005/Atom")))
                                {
                                    ServiceBusConnectionDetail entryInstance = new ServiceBusConnectionDetail();
                                    result.ConnectionDetails.Add(entryInstance);
                                    
                                    XElement contentElement = entriesElement.Element(XName.Get("content", "http://www.w3.org/2005/Atom"));
                                    if (contentElement != null)
                                    {
                                        XElement connectionDetailElement = contentElement.Element(XName.Get("ConnectionDetail", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                        if (connectionDetailElement != null)
                                        {
                                            XElement keyNameElement = connectionDetailElement.Element(XName.Get("KeyName", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                            if (keyNameElement != null)
                                            {
                                                string keyNameInstance = keyNameElement.Value;
                                                entryInstance.KeyName = keyNameInstance;
                                            }
                                            
                                            XElement connectionStringElement = connectionDetailElement.Element(XName.Get("ConnectionString", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                            if (connectionStringElement != null)
                                            {
                                                string connectionStringInstance = connectionStringElement.Value;
                                                entryInstance.ConnectionString = connectionStringInstance;
                                            }
                                            
                                            XElement authorizationTypeElement = connectionDetailElement.Element(XName.Get("AuthorizationType", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                            if (authorizationTypeElement != null)
                                            {
                                                string authorizationTypeInstance = authorizationTypeElement.Value;
                                                entryInstance.AuthorizationType = authorizationTypeInstance;
                                            }
                                            
                                            XElement rightsSequenceElement = connectionDetailElement.Element(XName.Get("Rights", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                            if (rightsSequenceElement != null)
                                            {
                                                foreach (XElement rightsElement in rightsSequenceElement.Elements(XName.Get("AccessRights", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect")))
                                                {
                                                    entryInstance.Rights.Add(((AccessRight)Enum.Parse(typeof(AccessRight), rightsElement.Value, true)));
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Enumerates the queues in the service namespace. The result is
        /// returned in pages, each containing up to 100 queues. If the
        /// namespace contains more than 100 queues, a feed is returned that
        /// contains the first page and a next link with the URI to view the
        /// next page of data.  (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/hh780759.asp
        /// for more information)
        /// </summary>
        /// <param name='namespaceName'>
        /// Required. The namespace name.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A response to a request for a list of queues.
        /// </returns>
        public async Task<ServiceBusQueuesResponse> ListAsync(string namespaceName, CancellationToken cancellationToken)
        {
            // Validate
            if (namespaceName == null)
            {
                throw new ArgumentNullException("namespaceName");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("namespaceName", namespaceName);
                TracingAdapter.Enter(invocationId, this, "ListAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/services/servicebus/namespaces/";
            url = url + Uri.EscapeDataString(namespaceName);
            url = url + "/Queues";
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Get;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("x-ms-version", "2013-08-01");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, null, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    ServiceBusQueuesResponse result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new ServiceBusQueuesResponse();
                        XDocument responseDoc = XDocument.Parse(responseContent);
                        
                        XElement feedElement = responseDoc.Element(XName.Get("feed", "http://www.w3.org/2005/Atom"));
                        if (feedElement != null)
                        {
                            if (feedElement != null)
                            {
                                foreach (XElement entriesElement in feedElement.Elements(XName.Get("entry", "http://www.w3.org/2005/Atom")))
                                {
                                    ServiceBusQueue entryInstance = new ServiceBusQueue();
                                    result.Queues.Add(entryInstance);
                                    
                                    XElement titleElement = entriesElement.Element(XName.Get("title", "http://www.w3.org/2005/Atom"));
                                    if (titleElement != null)
                                    {
                                        string titleInstance = titleElement.Value;
                                        entryInstance.Name = titleInstance;
                                    }
                                    
                                    XElement contentElement = entriesElement.Element(XName.Get("content", "http://www.w3.org/2005/Atom"));
                                    if (contentElement != null)
                                    {
                                        XElement queueDescriptionElement = contentElement.Element(XName.Get("QueueDescription", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                        if (queueDescriptionElement != null)
                                        {
                                            XElement lockDurationElement = queueDescriptionElement.Element(XName.Get("LockDuration", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                            if (lockDurationElement != null)
                                            {
                                                string lockDurationInstance = lockDurationElement.Value;
                                                entryInstance.LockDuration = lockDurationInstance;
                                            }
                                            
                                            XElement maxSizeInMegabytesElement = queueDescriptionElement.Element(XName.Get("MaxSizeInMegabytes", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                            if (maxSizeInMegabytesElement != null)
                                            {
                                                int maxSizeInMegabytesInstance = int.Parse(maxSizeInMegabytesElement.Value, CultureInfo.InvariantCulture);
                                                entryInstance.MaxSizeInMegabytes = maxSizeInMegabytesInstance;
                                            }
                                            
                                            XElement requiresDuplicateDetectionElement = queueDescriptionElement.Element(XName.Get("RequiresDuplicateDetection", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                            if (requiresDuplicateDetectionElement != null)
                                            {
                                                bool requiresDuplicateDetectionInstance = bool.Parse(requiresDuplicateDetectionElement.Value);
                                                entryInstance.RequiresDuplicateDetection = requiresDuplicateDetectionInstance;
                                            }
                                            
                                            XElement requiresSessionElement = queueDescriptionElement.Element(XName.Get("RequiresSession", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                            if (requiresSessionElement != null)
                                            {
                                                bool requiresSessionInstance = bool.Parse(requiresSessionElement.Value);
                                                entryInstance.RequiresSession = requiresSessionInstance;
                                            }
                                            
                                            XElement defaultMessageTimeToLiveElement = queueDescriptionElement.Element(XName.Get("DefaultMessageTimeToLive", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                            if (defaultMessageTimeToLiveElement != null)
                                            {
                                                string defaultMessageTimeToLiveInstance = defaultMessageTimeToLiveElement.Value;
                                                entryInstance.DefaultMessageTimeToLive = defaultMessageTimeToLiveInstance;
                                            }
                                            
                                            XElement deadLetteringOnMessageExpirationElement = queueDescriptionElement.Element(XName.Get("DeadLetteringOnMessageExpiration", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                            if (deadLetteringOnMessageExpirationElement != null)
                                            {
                                                bool deadLetteringOnMessageExpirationInstance = bool.Parse(deadLetteringOnMessageExpirationElement.Value);
                                                entryInstance.DeadLetteringOnMessageExpiration = deadLetteringOnMessageExpirationInstance;
                                            }
                                            
                                            XElement duplicateDetectionHistoryTimeWindowElement = queueDescriptionElement.Element(XName.Get("DuplicateDetectionHistoryTimeWindow", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                            if (duplicateDetectionHistoryTimeWindowElement != null)
                                            {
                                                string duplicateDetectionHistoryTimeWindowInstance = duplicateDetectionHistoryTimeWindowElement.Value;
                                                entryInstance.DuplicateDetectionHistoryTimeWindow = duplicateDetectionHistoryTimeWindowInstance;
                                            }
                                            
                                            XElement maxDeliveryCountElement = queueDescriptionElement.Element(XName.Get("MaxDeliveryCount", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                            if (maxDeliveryCountElement != null)
                                            {
                                                int maxDeliveryCountInstance = int.Parse(maxDeliveryCountElement.Value, CultureInfo.InvariantCulture);
                                                entryInstance.MaxDeliveryCount = maxDeliveryCountInstance;
                                            }
                                            
                                            XElement enableBatchedOperationsElement = queueDescriptionElement.Element(XName.Get("EnableBatchedOperations", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                            if (enableBatchedOperationsElement != null)
                                            {
                                                bool enableBatchedOperationsInstance = bool.Parse(enableBatchedOperationsElement.Value);
                                                entryInstance.EnableBatchedOperations = enableBatchedOperationsInstance;
                                            }
                                            
                                            XElement sizeInBytesElement = queueDescriptionElement.Element(XName.Get("SizeInBytes", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                            if (sizeInBytesElement != null)
                                            {
                                                int sizeInBytesInstance = int.Parse(sizeInBytesElement.Value, CultureInfo.InvariantCulture);
                                                entryInstance.SizeInBytes = sizeInBytesInstance;
                                            }
                                            
                                            XElement messageCountElement = queueDescriptionElement.Element(XName.Get("MessageCount", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                            if (messageCountElement != null)
                                            {
                                                int messageCountInstance = int.Parse(messageCountElement.Value, CultureInfo.InvariantCulture);
                                                entryInstance.MessageCount = messageCountInstance;
                                            }
                                            
                                            XElement isAnonymousAccessibleElement = queueDescriptionElement.Element(XName.Get("IsAnonymousAccessible", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                            if (isAnonymousAccessibleElement != null)
                                            {
                                                bool isAnonymousAccessibleInstance = bool.Parse(isAnonymousAccessibleElement.Value);
                                                entryInstance.IsAnonymousAccessible = isAnonymousAccessibleInstance;
                                            }
                                            
                                            XElement authorizationRulesSequenceElement = queueDescriptionElement.Element(XName.Get("AuthorizationRules", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                            if (authorizationRulesSequenceElement != null)
                                            {
                                                foreach (XElement authorizationRulesElement in authorizationRulesSequenceElement.Elements(XName.Get("AuthorizationRule", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect")))
                                                {
                                                    ServiceBusSharedAccessAuthorizationRule authorizationRuleInstance = new ServiceBusSharedAccessAuthorizationRule();
                                                    entryInstance.AuthorizationRules.Add(authorizationRuleInstance);
                                                    
                                                    XElement claimTypeElement = authorizationRulesElement.Element(XName.Get("ClaimType", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                                    if (claimTypeElement != null)
                                                    {
                                                        string claimTypeInstance = claimTypeElement.Value;
                                                        authorizationRuleInstance.ClaimType = claimTypeInstance;
                                                    }
                                                    
                                                    XElement claimValueElement = authorizationRulesElement.Element(XName.Get("ClaimValue", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                                    if (claimValueElement != null)
                                                    {
                                                        string claimValueInstance = claimValueElement.Value;
                                                        authorizationRuleInstance.ClaimValue = claimValueInstance;
                                                    }
                                                    
                                                    XElement rightsSequenceElement = authorizationRulesElement.Element(XName.Get("Rights", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                                    if (rightsSequenceElement != null)
                                                    {
                                                        foreach (XElement rightsElement in rightsSequenceElement.Elements(XName.Get("AccessRights", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect")))
                                                        {
                                                            authorizationRuleInstance.Rights.Add(((AccessRight)Enum.Parse(typeof(AccessRight), rightsElement.Value, true)));
                                                        }
                                                    }
                                                    
                                                    XElement createdTimeElement = authorizationRulesElement.Element(XName.Get("CreatedTime", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                                    if (createdTimeElement != null)
                                                    {
                                                        DateTime createdTimeInstance = DateTime.Parse(createdTimeElement.Value, CultureInfo.InvariantCulture);
                                                        authorizationRuleInstance.CreatedTime = createdTimeInstance;
                                                    }
                                                    
                                                    XElement keyNameElement = authorizationRulesElement.Element(XName.Get("KeyName", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                                    if (keyNameElement != null)
                                                    {
                                                        string keyNameInstance = keyNameElement.Value;
                                                        authorizationRuleInstance.KeyName = keyNameInstance;
                                                    }
                                                    
                                                    XElement modifiedTimeElement = authorizationRulesElement.Element(XName.Get("ModifiedTime", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                                    if (modifiedTimeElement != null)
                                                    {
                                                        DateTime modifiedTimeInstance = DateTime.Parse(modifiedTimeElement.Value, CultureInfo.InvariantCulture);
                                                        authorizationRuleInstance.ModifiedTime = modifiedTimeInstance;
                                                    }
                                                    
                                                    XElement primaryKeyElement = authorizationRulesElement.Element(XName.Get("PrimaryKey", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                                    if (primaryKeyElement != null)
                                                    {
                                                        string primaryKeyInstance = primaryKeyElement.Value;
                                                        authorizationRuleInstance.PrimaryKey = primaryKeyInstance;
                                                    }
                                                    
                                                    XElement secondaryKeyElement = authorizationRulesElement.Element(XName.Get("SecondaryKey", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                                    if (secondaryKeyElement != null)
                                                    {
                                                        string secondaryKeyInstance = secondaryKeyElement.Value;
                                                        authorizationRuleInstance.SecondaryKey = secondaryKeyInstance;
                                                    }
                                                }
                                            }
                                            
                                            XElement statusElement = queueDescriptionElement.Element(XName.Get("Status", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                            if (statusElement != null)
                                            {
                                                string statusInstance = statusElement.Value;
                                                entryInstance.Status = statusInstance;
                                            }
                                            
                                            XElement createdAtElement = queueDescriptionElement.Element(XName.Get("CreatedAt", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                            if (createdAtElement != null)
                                            {
                                                DateTime createdAtInstance = DateTime.Parse(createdAtElement.Value, CultureInfo.InvariantCulture);
                                                entryInstance.CreatedAt = createdAtInstance;
                                            }
                                            
                                            XElement updatedAtElement = queueDescriptionElement.Element(XName.Get("UpdatedAt", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                            if (updatedAtElement != null)
                                            {
                                                DateTime updatedAtInstance = DateTime.Parse(updatedAtElement.Value, CultureInfo.InvariantCulture);
                                                entryInstance.UpdatedAt = updatedAtInstance;
                                            }
                                            
                                            XElement accessedAtElement = queueDescriptionElement.Element(XName.Get("AccessedAt", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                            if (accessedAtElement != null)
                                            {
                                                DateTime accessedAtInstance = DateTime.Parse(accessedAtElement.Value, CultureInfo.InvariantCulture);
                                                entryInstance.AccessedAt = accessedAtInstance;
                                            }
                                            
                                            XElement supportOrderingElement = queueDescriptionElement.Element(XName.Get("SupportOrdering", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                            if (supportOrderingElement != null)
                                            {
                                                bool supportOrderingInstance = bool.Parse(supportOrderingElement.Value);
                                                entryInstance.SupportOrdering = supportOrderingInstance;
                                            }
                                            
                                            XElement countDetailsElement = queueDescriptionElement.Element(XName.Get("CountDetails", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                            if (countDetailsElement != null)
                                            {
                                                CountDetails countDetailsInstance = new CountDetails();
                                                entryInstance.CountDetails = countDetailsInstance;
                                                
                                                XElement activeMessageCountElement = countDetailsElement.Element(XName.Get("ActiveMessageCount", "http://schemas.microsoft.com/netservices/2011/06/servicebus"));
                                                if (activeMessageCountElement != null)
                                                {
                                                    int activeMessageCountInstance = int.Parse(activeMessageCountElement.Value, CultureInfo.InvariantCulture);
                                                    countDetailsInstance.ActiveMessageCount = activeMessageCountInstance;
                                                }
                                                
                                                XElement deadLetterMessageCountElement = countDetailsElement.Element(XName.Get("DeadLetterMessageCount", "http://schemas.microsoft.com/netservices/2011/06/servicebus"));
                                                if (deadLetterMessageCountElement != null)
                                                {
                                                    int deadLetterMessageCountInstance = int.Parse(deadLetterMessageCountElement.Value, CultureInfo.InvariantCulture);
                                                    countDetailsInstance.DeadLetterMessageCount = deadLetterMessageCountInstance;
                                                }
                                                
                                                XElement scheduledMessageCountElement = countDetailsElement.Element(XName.Get("ScheduledMessageCount", "http://schemas.microsoft.com/netservices/2011/06/servicebus"));
                                                if (scheduledMessageCountElement != null)
                                                {
                                                    int scheduledMessageCountInstance = int.Parse(scheduledMessageCountElement.Value, CultureInfo.InvariantCulture);
                                                    countDetailsInstance.ScheduledMessageCount = scheduledMessageCountInstance;
                                                }
                                                
                                                XElement transferDeadLetterMessageCountElement = countDetailsElement.Element(XName.Get("TransferDeadLetterMessageCount", "http://schemas.microsoft.com/netservices/2011/06/servicebus"));
                                                if (transferDeadLetterMessageCountElement != null)
                                                {
                                                    int transferDeadLetterMessageCountInstance = int.Parse(transferDeadLetterMessageCountElement.Value, CultureInfo.InvariantCulture);
                                                    countDetailsInstance.TransferDeadLetterMessageCount = transferDeadLetterMessageCountInstance;
                                                }
                                                
                                                XElement transferMessageCountElement = countDetailsElement.Element(XName.Get("TransferMessageCount", "http://schemas.microsoft.com/netservices/2011/06/servicebus"));
                                                if (transferMessageCountElement != null)
                                                {
                                                    int transferMessageCountInstance = int.Parse(transferMessageCountElement.Value, CultureInfo.InvariantCulture);
                                                    countDetailsInstance.TransferMessageCount = transferMessageCountInstance;
                                                }
                                            }
                                            
                                            XElement autoDeleteOnIdleElement = queueDescriptionElement.Element(XName.Get("AutoDeleteOnIdle", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                            if (autoDeleteOnIdleElement != null)
                                            {
                                                string autoDeleteOnIdleInstance = autoDeleteOnIdleElement.Value;
                                                entryInstance.AutoDeleteOnIdle = autoDeleteOnIdleInstance;
                                            }
                                            
                                            XElement entityAvailabilityStatusElement = queueDescriptionElement.Element(XName.Get("EntityAvailabilityStatus", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                            if (entityAvailabilityStatusElement != null)
                                            {
                                                string entityAvailabilityStatusInstance = entityAvailabilityStatusElement.Value;
                                                entryInstance.EntityAvailabilityStatus = entityAvailabilityStatusInstance;
                                            }
                                        }
                                    }
                                }
                            }
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
        
        /// <summary>
        /// Updates the queue description and makes a call to update
        /// corresponding DB entries.  (see
        /// http://msdn.microsoft.com/en-us/library/windowsazure/jj856305.aspx
        /// for more information)
        /// </summary>
        /// <param name='namespaceName'>
        /// Required. The namespace name.
        /// </param>
        /// <param name='queue'>
        /// Required. The service bus queue.
        /// </param>
        /// <param name='cancellationToken'>
        /// Cancellation token.
        /// </param>
        /// <returns>
        /// A response to a request for a particular queue.
        /// </returns>
        public async Task<ServiceBusQueueResponse> UpdateAsync(string namespaceName, ServiceBusQueue queue, CancellationToken cancellationToken)
        {
            // Validate
            if (namespaceName == null)
            {
                throw new ArgumentNullException("namespaceName");
            }
            if (queue == null)
            {
                throw new ArgumentNullException("queue");
            }
            
            // Tracing
            bool shouldTrace = TracingAdapter.IsEnabled;
            string invocationId = null;
            if (shouldTrace)
            {
                invocationId = TracingAdapter.NextInvocationId.ToString();
                Dictionary<string, object> tracingParameters = new Dictionary<string, object>();
                tracingParameters.Add("namespaceName", namespaceName);
                tracingParameters.Add("queue", queue);
                TracingAdapter.Enter(invocationId, this, "UpdateAsync", tracingParameters);
            }
            
            // Construct URL
            string url = "";
            url = url + "/";
            if (this.Client.Credentials.SubscriptionId != null)
            {
                url = url + Uri.EscapeDataString(this.Client.Credentials.SubscriptionId);
            }
            url = url + "/services/servicebus/namespaces/";
            url = url + Uri.EscapeDataString(namespaceName);
            url = url + "/queues/";
            if (queue.Name != null)
            {
                url = url + Uri.EscapeDataString(queue.Name);
            }
            url = url + "/";
            string baseUrl = this.Client.BaseUri.AbsoluteUri;
            // Trim '/' character from the end of baseUrl and beginning of url.
            if (baseUrl[baseUrl.Length - 1] == '/')
            {
                baseUrl = baseUrl.Substring(0, baseUrl.Length - 1);
            }
            if (url[0] == '/')
            {
                url = url.Substring(1);
            }
            url = baseUrl + "/" + url;
            url = url.Replace(" ", "%20");
            
            // Create HTTP transport objects
            HttpRequestMessage httpRequest = null;
            try
            {
                httpRequest = new HttpRequestMessage();
                httpRequest.Method = HttpMethod.Put;
                httpRequest.RequestUri = new Uri(url);
                
                // Set Headers
                httpRequest.Headers.Add("if-match", "*");
                httpRequest.Headers.Add("type", "entry");
                httpRequest.Headers.Add("x-ms-version", "2013-08-01");
                httpRequest.Headers.Add("x-process-at", "ServiceBus");
                
                // Set Credentials
                cancellationToken.ThrowIfCancellationRequested();
                await this.Client.Credentials.ProcessHttpRequestAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                
                // Serialize Request
                string requestContent = null;
                XDocument requestDoc = new XDocument();
                
                XElement entryElement = new XElement(XName.Get("entry", "http://www.w3.org/2005/Atom"));
                requestDoc.Add(entryElement);
                
                XElement contentElement = new XElement(XName.Get("content", "http://www.w3.org/2005/Atom"));
                entryElement.Add(contentElement);
                
                XAttribute typeAttribute = new XAttribute(XName.Get("type", ""), "");
                typeAttribute.Value = "application/atom+xml;type=entry;charset=utf-8";
                contentElement.Add(typeAttribute);
                
                XElement queueDescriptionElement = new XElement(XName.Get("QueueDescription", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                contentElement.Add(queueDescriptionElement);
                
                if (queue.LockDuration != null)
                {
                    XElement lockDurationElement = new XElement(XName.Get("LockDuration", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                    lockDurationElement.Value = queue.LockDuration;
                    queueDescriptionElement.Add(lockDurationElement);
                }
                
                XElement maxSizeInMegabytesElement = new XElement(XName.Get("MaxSizeInMegabytes", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                maxSizeInMegabytesElement.Value = queue.MaxSizeInMegabytes.ToString();
                queueDescriptionElement.Add(maxSizeInMegabytesElement);
                
                XElement requiresDuplicateDetectionElement = new XElement(XName.Get("RequiresDuplicateDetection", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                requiresDuplicateDetectionElement.Value = queue.RequiresDuplicateDetection.ToString().ToLower();
                queueDescriptionElement.Add(requiresDuplicateDetectionElement);
                
                XElement requiresSessionElement = new XElement(XName.Get("RequiresSession", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                requiresSessionElement.Value = queue.RequiresSession.ToString().ToLower();
                queueDescriptionElement.Add(requiresSessionElement);
                
                if (queue.DefaultMessageTimeToLive != null)
                {
                    XElement defaultMessageTimeToLiveElement = new XElement(XName.Get("DefaultMessageTimeToLive", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                    defaultMessageTimeToLiveElement.Value = queue.DefaultMessageTimeToLive;
                    queueDescriptionElement.Add(defaultMessageTimeToLiveElement);
                }
                
                XElement deadLetteringOnMessageExpirationElement = new XElement(XName.Get("DeadLetteringOnMessageExpiration", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                deadLetteringOnMessageExpirationElement.Value = queue.DeadLetteringOnMessageExpiration.ToString().ToLower();
                queueDescriptionElement.Add(deadLetteringOnMessageExpirationElement);
                
                if (queue.DuplicateDetectionHistoryTimeWindow != null)
                {
                    XElement duplicateDetectionHistoryTimeWindowElement = new XElement(XName.Get("DuplicateDetectionHistoryTimeWindow", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                    duplicateDetectionHistoryTimeWindowElement.Value = queue.DuplicateDetectionHistoryTimeWindow;
                    queueDescriptionElement.Add(duplicateDetectionHistoryTimeWindowElement);
                }
                
                XElement enableBatchedOperationsElement = new XElement(XName.Get("EnableBatchedOperations", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                enableBatchedOperationsElement.Value = queue.EnableBatchedOperations.ToString().ToLower();
                queueDescriptionElement.Add(enableBatchedOperationsElement);
                
                XElement sizeInBytesElement = new XElement(XName.Get("SizeInBytes", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                sizeInBytesElement.Value = queue.SizeInBytes.ToString();
                queueDescriptionElement.Add(sizeInBytesElement);
                
                XElement messageCountElement = new XElement(XName.Get("MessageCount", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                messageCountElement.Value = queue.MessageCount.ToString();
                queueDescriptionElement.Add(messageCountElement);
                
                XElement isAnonymousAccessibleElement = new XElement(XName.Get("IsAnonymousAccessible", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                isAnonymousAccessibleElement.Value = queue.IsAnonymousAccessible.ToString().ToLower();
                queueDescriptionElement.Add(isAnonymousAccessibleElement);
                
                if (queue.AuthorizationRules != null)
                {
                    if (queue.AuthorizationRules is ILazyCollection == false || ((ILazyCollection)queue.AuthorizationRules).IsInitialized)
                    {
                        XElement authorizationRulesSequenceElement = new XElement(XName.Get("AuthorizationRules", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                        foreach (ServiceBusSharedAccessAuthorizationRule authorizationRulesItem in queue.AuthorizationRules)
                        {
                            XElement authorizationRuleElement = new XElement(XName.Get("AuthorizationRule", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                            authorizationRulesSequenceElement.Add(authorizationRuleElement);
                            
                            XAttribute typeAttribute2 = new XAttribute(XName.Get("type", "http://www.w3.org/2001/XMLSchema-instance"), "");
                            typeAttribute2.Value = "SharedAccessAuthorizationRule";
                            authorizationRuleElement.Add(typeAttribute2);
                            
                            if (authorizationRulesItem.ClaimType != null)
                            {
                                XElement claimTypeElement = new XElement(XName.Get("ClaimType", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                claimTypeElement.Value = authorizationRulesItem.ClaimType;
                                authorizationRuleElement.Add(claimTypeElement);
                            }
                            
                            if (authorizationRulesItem.ClaimValue != null)
                            {
                                XElement claimValueElement = new XElement(XName.Get("ClaimValue", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                claimValueElement.Value = authorizationRulesItem.ClaimValue;
                                authorizationRuleElement.Add(claimValueElement);
                            }
                            
                            if (authorizationRulesItem.Rights != null)
                            {
                                if (authorizationRulesItem.Rights is ILazyCollection == false || ((ILazyCollection)authorizationRulesItem.Rights).IsInitialized)
                                {
                                    XElement rightsSequenceElement = new XElement(XName.Get("Rights", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    foreach (AccessRight rightsItem in authorizationRulesItem.Rights)
                                    {
                                        XElement rightsItemElement = new XElement(XName.Get("AccessRights", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                        rightsItemElement.Value = rightsItem.ToString();
                                        rightsSequenceElement.Add(rightsItemElement);
                                    }
                                    authorizationRuleElement.Add(rightsSequenceElement);
                                }
                            }
                            
                            XElement createdTimeElement = new XElement(XName.Get("CreatedTime", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                            createdTimeElement.Value = authorizationRulesItem.CreatedTime.ToString();
                            authorizationRuleElement.Add(createdTimeElement);
                            
                            if (authorizationRulesItem.KeyName != null)
                            {
                                XElement keyNameElement = new XElement(XName.Get("KeyName", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                keyNameElement.Value = authorizationRulesItem.KeyName;
                                authorizationRuleElement.Add(keyNameElement);
                            }
                            
                            XElement modifiedTimeElement = new XElement(XName.Get("ModifiedTime", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                            modifiedTimeElement.Value = authorizationRulesItem.ModifiedTime.ToString();
                            authorizationRuleElement.Add(modifiedTimeElement);
                            
                            if (authorizationRulesItem.PrimaryKey != null)
                            {
                                XElement primaryKeyElement = new XElement(XName.Get("PrimaryKey", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                primaryKeyElement.Value = authorizationRulesItem.PrimaryKey;
                                authorizationRuleElement.Add(primaryKeyElement);
                            }
                            
                            if (authorizationRulesItem.SecondaryKey != null)
                            {
                                XElement secondaryKeyElement = new XElement(XName.Get("SecondaryKey", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                secondaryKeyElement.Value = authorizationRulesItem.SecondaryKey;
                                authorizationRuleElement.Add(secondaryKeyElement);
                            }
                        }
                        queueDescriptionElement.Add(authorizationRulesSequenceElement);
                    }
                }
                
                if (queue.Status != null)
                {
                    XElement statusElement = new XElement(XName.Get("Status", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                    statusElement.Value = queue.Status;
                    queueDescriptionElement.Add(statusElement);
                }
                
                XElement supportOrderingElement = new XElement(XName.Get("SupportOrdering", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                supportOrderingElement.Value = queue.SupportOrdering.ToString().ToLower();
                queueDescriptionElement.Add(supportOrderingElement);
                
                if (queue.CountDetails != null)
                {
                    XElement countDetailsElement = new XElement(XName.Get("CountDetails", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                    queueDescriptionElement.Add(countDetailsElement);
                    
                    XElement activeMessageCountElement = new XElement(XName.Get("ActiveMessageCount", "http://schemas.microsoft.com/netservices/2011/06/servicebus"));
                    activeMessageCountElement.Value = queue.CountDetails.ActiveMessageCount.ToString();
                    countDetailsElement.Add(activeMessageCountElement);
                    
                    XElement deadLetterMessageCountElement = new XElement(XName.Get("DeadLetterMessageCount", "http://schemas.microsoft.com/netservices/2011/06/servicebus"));
                    deadLetterMessageCountElement.Value = queue.CountDetails.DeadLetterMessageCount.ToString();
                    countDetailsElement.Add(deadLetterMessageCountElement);
                    
                    XElement scheduledMessageCountElement = new XElement(XName.Get("ScheduledMessageCount", "http://schemas.microsoft.com/netservices/2011/06/servicebus"));
                    scheduledMessageCountElement.Value = queue.CountDetails.ScheduledMessageCount.ToString();
                    countDetailsElement.Add(scheduledMessageCountElement);
                    
                    XElement transferDeadLetterMessageCountElement = new XElement(XName.Get("TransferDeadLetterMessageCount", "http://schemas.microsoft.com/netservices/2011/06/servicebus"));
                    transferDeadLetterMessageCountElement.Value = queue.CountDetails.TransferDeadLetterMessageCount.ToString();
                    countDetailsElement.Add(transferDeadLetterMessageCountElement);
                    
                    XElement transferMessageCountElement = new XElement(XName.Get("TransferMessageCount", "http://schemas.microsoft.com/netservices/2011/06/servicebus"));
                    transferMessageCountElement.Value = queue.CountDetails.TransferMessageCount.ToString();
                    countDetailsElement.Add(transferMessageCountElement);
                }
                
                if (queue.AutoDeleteOnIdle != null)
                {
                    XElement autoDeleteOnIdleElement = new XElement(XName.Get("AutoDeleteOnIdle", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                    autoDeleteOnIdleElement.Value = queue.AutoDeleteOnIdle;
                    queueDescriptionElement.Add(autoDeleteOnIdleElement);
                }
                
                if (queue.EntityAvailabilityStatus != null)
                {
                    XElement entityAvailabilityStatusElement = new XElement(XName.Get("EntityAvailabilityStatus", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                    entityAvailabilityStatusElement.Value = queue.EntityAvailabilityStatus;
                    queueDescriptionElement.Add(entityAvailabilityStatusElement);
                }
                
                requestContent = requestDoc.ToString();
                httpRequest.Content = new StringContent(requestContent, Encoding.UTF8);
                httpRequest.Content.Headers.ContentType = MediaTypeHeaderValue.Parse("application/atom+xml");
                
                // Send Request
                HttpResponseMessage httpResponse = null;
                try
                {
                    if (shouldTrace)
                    {
                        TracingAdapter.SendRequest(invocationId, httpRequest);
                    }
                    cancellationToken.ThrowIfCancellationRequested();
                    httpResponse = await this.Client.HttpClient.SendAsync(httpRequest, cancellationToken).ConfigureAwait(false);
                    if (shouldTrace)
                    {
                        TracingAdapter.ReceiveResponse(invocationId, httpResponse);
                    }
                    HttpStatusCode statusCode = httpResponse.StatusCode;
                    if (statusCode != HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        CloudException ex = CloudException.Create(httpRequest, requestContent, httpResponse, await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false));
                        if (shouldTrace)
                        {
                            TracingAdapter.Error(invocationId, ex);
                        }
                        throw ex;
                    }
                    
                    // Create Result
                    ServiceBusQueueResponse result = null;
                    // Deserialize Response
                    if (statusCode == HttpStatusCode.OK)
                    {
                        cancellationToken.ThrowIfCancellationRequested();
                        string responseContent = await httpResponse.Content.ReadAsStringAsync().ConfigureAwait(false);
                        result = new ServiceBusQueueResponse();
                        XDocument responseDoc = XDocument.Parse(responseContent);
                        
                        XElement entryElement2 = responseDoc.Element(XName.Get("entry", "http://www.w3.org/2005/Atom"));
                        if (entryElement2 != null)
                        {
                            XElement titleElement = entryElement2.Element(XName.Get("title", "http://www.w3.org/2005/Atom"));
                            if (titleElement != null)
                            {
                            }
                            
                            XElement contentElement2 = entryElement2.Element(XName.Get("content", "http://www.w3.org/2005/Atom"));
                            if (contentElement2 != null)
                            {
                                XElement queueDescriptionElement2 = contentElement2.Element(XName.Get("QueueDescription", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                if (queueDescriptionElement2 != null)
                                {
                                    ServiceBusQueue queueDescriptionInstance = new ServiceBusQueue();
                                    result.Queue = queueDescriptionInstance;
                                    
                                    XElement lockDurationElement2 = queueDescriptionElement2.Element(XName.Get("LockDuration", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (lockDurationElement2 != null)
                                    {
                                        string lockDurationInstance = lockDurationElement2.Value;
                                        queueDescriptionInstance.LockDuration = lockDurationInstance;
                                    }
                                    
                                    XElement maxSizeInMegabytesElement2 = queueDescriptionElement2.Element(XName.Get("MaxSizeInMegabytes", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (maxSizeInMegabytesElement2 != null)
                                    {
                                        int maxSizeInMegabytesInstance = int.Parse(maxSizeInMegabytesElement2.Value, CultureInfo.InvariantCulture);
                                        queueDescriptionInstance.MaxSizeInMegabytes = maxSizeInMegabytesInstance;
                                    }
                                    
                                    XElement requiresDuplicateDetectionElement2 = queueDescriptionElement2.Element(XName.Get("RequiresDuplicateDetection", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (requiresDuplicateDetectionElement2 != null)
                                    {
                                        bool requiresDuplicateDetectionInstance = bool.Parse(requiresDuplicateDetectionElement2.Value);
                                        queueDescriptionInstance.RequiresDuplicateDetection = requiresDuplicateDetectionInstance;
                                    }
                                    
                                    XElement requiresSessionElement2 = queueDescriptionElement2.Element(XName.Get("RequiresSession", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (requiresSessionElement2 != null)
                                    {
                                        bool requiresSessionInstance = bool.Parse(requiresSessionElement2.Value);
                                        queueDescriptionInstance.RequiresSession = requiresSessionInstance;
                                    }
                                    
                                    XElement defaultMessageTimeToLiveElement2 = queueDescriptionElement2.Element(XName.Get("DefaultMessageTimeToLive", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (defaultMessageTimeToLiveElement2 != null)
                                    {
                                        string defaultMessageTimeToLiveInstance = defaultMessageTimeToLiveElement2.Value;
                                        queueDescriptionInstance.DefaultMessageTimeToLive = defaultMessageTimeToLiveInstance;
                                    }
                                    
                                    XElement deadLetteringOnMessageExpirationElement2 = queueDescriptionElement2.Element(XName.Get("DeadLetteringOnMessageExpiration", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (deadLetteringOnMessageExpirationElement2 != null)
                                    {
                                        bool deadLetteringOnMessageExpirationInstance = bool.Parse(deadLetteringOnMessageExpirationElement2.Value);
                                        queueDescriptionInstance.DeadLetteringOnMessageExpiration = deadLetteringOnMessageExpirationInstance;
                                    }
                                    
                                    XElement duplicateDetectionHistoryTimeWindowElement2 = queueDescriptionElement2.Element(XName.Get("DuplicateDetectionHistoryTimeWindow", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (duplicateDetectionHistoryTimeWindowElement2 != null)
                                    {
                                        string duplicateDetectionHistoryTimeWindowInstance = duplicateDetectionHistoryTimeWindowElement2.Value;
                                        queueDescriptionInstance.DuplicateDetectionHistoryTimeWindow = duplicateDetectionHistoryTimeWindowInstance;
                                    }
                                    
                                    XElement maxDeliveryCountElement = queueDescriptionElement2.Element(XName.Get("MaxDeliveryCount", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (maxDeliveryCountElement != null)
                                    {
                                        int maxDeliveryCountInstance = int.Parse(maxDeliveryCountElement.Value, CultureInfo.InvariantCulture);
                                        queueDescriptionInstance.MaxDeliveryCount = maxDeliveryCountInstance;
                                    }
                                    
                                    XElement enableBatchedOperationsElement2 = queueDescriptionElement2.Element(XName.Get("EnableBatchedOperations", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (enableBatchedOperationsElement2 != null)
                                    {
                                        bool enableBatchedOperationsInstance = bool.Parse(enableBatchedOperationsElement2.Value);
                                        queueDescriptionInstance.EnableBatchedOperations = enableBatchedOperationsInstance;
                                    }
                                    
                                    XElement sizeInBytesElement2 = queueDescriptionElement2.Element(XName.Get("SizeInBytes", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (sizeInBytesElement2 != null)
                                    {
                                        int sizeInBytesInstance = int.Parse(sizeInBytesElement2.Value, CultureInfo.InvariantCulture);
                                        queueDescriptionInstance.SizeInBytes = sizeInBytesInstance;
                                    }
                                    
                                    XElement messageCountElement2 = queueDescriptionElement2.Element(XName.Get("MessageCount", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (messageCountElement2 != null)
                                    {
                                        int messageCountInstance = int.Parse(messageCountElement2.Value, CultureInfo.InvariantCulture);
                                        queueDescriptionInstance.MessageCount = messageCountInstance;
                                    }
                                    
                                    XElement isAnonymousAccessibleElement2 = queueDescriptionElement2.Element(XName.Get("IsAnonymousAccessible", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (isAnonymousAccessibleElement2 != null)
                                    {
                                        bool isAnonymousAccessibleInstance = bool.Parse(isAnonymousAccessibleElement2.Value);
                                        queueDescriptionInstance.IsAnonymousAccessible = isAnonymousAccessibleInstance;
                                    }
                                    
                                    XElement authorizationRulesSequenceElement2 = queueDescriptionElement2.Element(XName.Get("AuthorizationRules", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (authorizationRulesSequenceElement2 != null)
                                    {
                                        foreach (XElement authorizationRulesElement in authorizationRulesSequenceElement2.Elements(XName.Get("AuthorizationRule", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect")))
                                        {
                                            ServiceBusSharedAccessAuthorizationRule authorizationRuleInstance = new ServiceBusSharedAccessAuthorizationRule();
                                            queueDescriptionInstance.AuthorizationRules.Add(authorizationRuleInstance);
                                            
                                            XElement claimTypeElement2 = authorizationRulesElement.Element(XName.Get("ClaimType", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                            if (claimTypeElement2 != null)
                                            {
                                                string claimTypeInstance = claimTypeElement2.Value;
                                                authorizationRuleInstance.ClaimType = claimTypeInstance;
                                            }
                                            
                                            XElement claimValueElement2 = authorizationRulesElement.Element(XName.Get("ClaimValue", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                            if (claimValueElement2 != null)
                                            {
                                                string claimValueInstance = claimValueElement2.Value;
                                                authorizationRuleInstance.ClaimValue = claimValueInstance;
                                            }
                                            
                                            XElement rightsSequenceElement2 = authorizationRulesElement.Element(XName.Get("Rights", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                            if (rightsSequenceElement2 != null)
                                            {
                                                foreach (XElement rightsElement in rightsSequenceElement2.Elements(XName.Get("AccessRights", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect")))
                                                {
                                                    authorizationRuleInstance.Rights.Add(((AccessRight)Enum.Parse(typeof(AccessRight), rightsElement.Value, true)));
                                                }
                                            }
                                            
                                            XElement createdTimeElement2 = authorizationRulesElement.Element(XName.Get("CreatedTime", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                            if (createdTimeElement2 != null)
                                            {
                                                DateTime createdTimeInstance = DateTime.Parse(createdTimeElement2.Value, CultureInfo.InvariantCulture);
                                                authorizationRuleInstance.CreatedTime = createdTimeInstance;
                                            }
                                            
                                            XElement keyNameElement2 = authorizationRulesElement.Element(XName.Get("KeyName", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                            if (keyNameElement2 != null)
                                            {
                                                string keyNameInstance = keyNameElement2.Value;
                                                authorizationRuleInstance.KeyName = keyNameInstance;
                                            }
                                            
                                            XElement modifiedTimeElement2 = authorizationRulesElement.Element(XName.Get("ModifiedTime", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                            if (modifiedTimeElement2 != null)
                                            {
                                                DateTime modifiedTimeInstance = DateTime.Parse(modifiedTimeElement2.Value, CultureInfo.InvariantCulture);
                                                authorizationRuleInstance.ModifiedTime = modifiedTimeInstance;
                                            }
                                            
                                            XElement primaryKeyElement2 = authorizationRulesElement.Element(XName.Get("PrimaryKey", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                            if (primaryKeyElement2 != null)
                                            {
                                                string primaryKeyInstance = primaryKeyElement2.Value;
                                                authorizationRuleInstance.PrimaryKey = primaryKeyInstance;
                                            }
                                            
                                            XElement secondaryKeyElement2 = authorizationRulesElement.Element(XName.Get("SecondaryKey", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                            if (secondaryKeyElement2 != null)
                                            {
                                                string secondaryKeyInstance = secondaryKeyElement2.Value;
                                                authorizationRuleInstance.SecondaryKey = secondaryKeyInstance;
                                            }
                                        }
                                    }
                                    
                                    XElement statusElement2 = queueDescriptionElement2.Element(XName.Get("Status", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (statusElement2 != null)
                                    {
                                        string statusInstance = statusElement2.Value;
                                        queueDescriptionInstance.Status = statusInstance;
                                    }
                                    
                                    XElement createdAtElement = queueDescriptionElement2.Element(XName.Get("CreatedAt", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (createdAtElement != null)
                                    {
                                        DateTime createdAtInstance = DateTime.Parse(createdAtElement.Value, CultureInfo.InvariantCulture);
                                        queueDescriptionInstance.CreatedAt = createdAtInstance;
                                    }
                                    
                                    XElement updatedAtElement = queueDescriptionElement2.Element(XName.Get("UpdatedAt", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (updatedAtElement != null)
                                    {
                                        DateTime updatedAtInstance = DateTime.Parse(updatedAtElement.Value, CultureInfo.InvariantCulture);
                                        queueDescriptionInstance.UpdatedAt = updatedAtInstance;
                                    }
                                    
                                    XElement accessedAtElement = queueDescriptionElement2.Element(XName.Get("AccessedAt", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (accessedAtElement != null)
                                    {
                                        DateTime accessedAtInstance = DateTime.Parse(accessedAtElement.Value, CultureInfo.InvariantCulture);
                                        queueDescriptionInstance.AccessedAt = accessedAtInstance;
                                    }
                                    
                                    XElement supportOrderingElement2 = queueDescriptionElement2.Element(XName.Get("SupportOrdering", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (supportOrderingElement2 != null)
                                    {
                                        bool supportOrderingInstance = bool.Parse(supportOrderingElement2.Value);
                                        queueDescriptionInstance.SupportOrdering = supportOrderingInstance;
                                    }
                                    
                                    XElement countDetailsElement2 = queueDescriptionElement2.Element(XName.Get("CountDetails", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (countDetailsElement2 != null)
                                    {
                                        CountDetails countDetailsInstance = new CountDetails();
                                        queueDescriptionInstance.CountDetails = countDetailsInstance;
                                        
                                        XElement activeMessageCountElement2 = countDetailsElement2.Element(XName.Get("ActiveMessageCount", "http://schemas.microsoft.com/netservices/2011/06/servicebus"));
                                        if (activeMessageCountElement2 != null)
                                        {
                                            int activeMessageCountInstance = int.Parse(activeMessageCountElement2.Value, CultureInfo.InvariantCulture);
                                            countDetailsInstance.ActiveMessageCount = activeMessageCountInstance;
                                        }
                                        
                                        XElement deadLetterMessageCountElement2 = countDetailsElement2.Element(XName.Get("DeadLetterMessageCount", "http://schemas.microsoft.com/netservices/2011/06/servicebus"));
                                        if (deadLetterMessageCountElement2 != null)
                                        {
                                            int deadLetterMessageCountInstance = int.Parse(deadLetterMessageCountElement2.Value, CultureInfo.InvariantCulture);
                                            countDetailsInstance.DeadLetterMessageCount = deadLetterMessageCountInstance;
                                        }
                                        
                                        XElement scheduledMessageCountElement2 = countDetailsElement2.Element(XName.Get("ScheduledMessageCount", "http://schemas.microsoft.com/netservices/2011/06/servicebus"));
                                        if (scheduledMessageCountElement2 != null)
                                        {
                                            int scheduledMessageCountInstance = int.Parse(scheduledMessageCountElement2.Value, CultureInfo.InvariantCulture);
                                            countDetailsInstance.ScheduledMessageCount = scheduledMessageCountInstance;
                                        }
                                        
                                        XElement transferDeadLetterMessageCountElement2 = countDetailsElement2.Element(XName.Get("TransferDeadLetterMessageCount", "http://schemas.microsoft.com/netservices/2011/06/servicebus"));
                                        if (transferDeadLetterMessageCountElement2 != null)
                                        {
                                            int transferDeadLetterMessageCountInstance = int.Parse(transferDeadLetterMessageCountElement2.Value, CultureInfo.InvariantCulture);
                                            countDetailsInstance.TransferDeadLetterMessageCount = transferDeadLetterMessageCountInstance;
                                        }
                                        
                                        XElement transferMessageCountElement2 = countDetailsElement2.Element(XName.Get("TransferMessageCount", "http://schemas.microsoft.com/netservices/2011/06/servicebus"));
                                        if (transferMessageCountElement2 != null)
                                        {
                                            int transferMessageCountInstance = int.Parse(transferMessageCountElement2.Value, CultureInfo.InvariantCulture);
                                            countDetailsInstance.TransferMessageCount = transferMessageCountInstance;
                                        }
                                    }
                                    
                                    XElement autoDeleteOnIdleElement2 = queueDescriptionElement2.Element(XName.Get("AutoDeleteOnIdle", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (autoDeleteOnIdleElement2 != null)
                                    {
                                        string autoDeleteOnIdleInstance = autoDeleteOnIdleElement2.Value;
                                        queueDescriptionInstance.AutoDeleteOnIdle = autoDeleteOnIdleInstance;
                                    }
                                    
                                    XElement entityAvailabilityStatusElement2 = queueDescriptionElement2.Element(XName.Get("EntityAvailabilityStatus", "http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"));
                                    if (entityAvailabilityStatusElement2 != null)
                                    {
                                        string entityAvailabilityStatusInstance = entityAvailabilityStatusElement2.Value;
                                        queueDescriptionInstance.EntityAvailabilityStatus = entityAvailabilityStatusInstance;
                                    }
                                }
                            }
                        }
                        
                    }
                    result.StatusCode = statusCode;
                    if (httpResponse.Headers.Contains("x-ms-request-id"))
                    {
                        result.RequestId = httpResponse.Headers.GetValues("x-ms-request-id").FirstOrDefault();
                    }
                    
                    if (shouldTrace)
                    {
                        TracingAdapter.Exit(invocationId, result);
                    }
                    return result;
                }
                finally
                {
                    if (httpResponse != null)
                    {
                        httpResponse.Dispose();
                    }
                }
            }
            finally
            {
                if (httpRequest != null)
                {
                    httpRequest.Dispose();
                }
            }
        }
    }
}
